from .interaction_dataset_reader import bark_state_from_motion_state
from .interaction_dataset_reader import trajectory_from_track
from .interaction_dataset_reader import shape_from_track
from .interaction_dataset_reader import init_state_from_track
from .interaction_dataset_reader import goal_definition_from_track
from .interaction_dataset_reader import behavior_from_track
from .interaction_dataset_reader import track_from_trackfile
from .interaction_dataset_reader import agent_from_trackfile
from .dataset_decomposer import  DatasetDecomposer

__all__ = ["DatasetDecomposer",
          "bark_state_from_motion_state",
          "trajectory_from_track",
          "shape_from_track",
          "init_state_from_track",
          "goal_definition_from_track",
          "behavior_from_track",
          "track_from_trackfile",
          "agent_from_trackfile"]



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import unittest
import os
import numpy as np
import matplotlib as mpl
if os.environ.get('DISPLAY', '') == '':
  print('no display found. Using non-interactive Agg backend')
  mpl.use('Agg')
from bark.core.world.agent import Agent
from bark.core.models.behavior import BehaviorConstantVelocity
from bark.core.models.dynamic import SingleTrackModel
from bark.core.models.execution import ExecutionModelInterpolate
from bark.core.geometry import Polygon2d, Point2d
from bark.core.geometry.standard_shapes import CarLimousine
from bark.runtime.commons import ParameterServer


class AgentTests(unittest.TestCase):
  def test_write_params_agent(self):
    params = ParameterServer()
    behavior = BehaviorConstantVelocity(params)
    execution = ExecutionModelInterpolate(params)
    dynamic = SingleTrackModel(params)
    shape = Polygon2d([1.25, 1, 0], [
        Point2d(0, 0),
        Point2d(0, 2),
        Point2d(4, 2),
        Point2d(4, 0),
        Point2d(0, 0)
    ])
    init_state = np.zeros(4)
    agent = Agent(init_state, behavior, dynamic, execution, shape,
                  params.AddChild("agent"))
    params.Save("written_agents_param_test.json")

  def test_draw_agents(self):
    params = ParameterServer()
    behavior = BehaviorConstantVelocity(params)
    execution = ExecutionModelInterpolate(params)
    dynamic = SingleTrackModel(params)
    shape = Polygon2d([1.25, 1, 0], [
        Point2d(0, 0),
        Point2d(0, 2),
        Point2d(4, 2),
        Point2d(4, 0),
        Point2d(0, 0)
    ])
    shape2 = CarLimousine()

    init_state = [0, 3, 2, 1]
    init_state2 = [0, 0, 5, 4]

    agent = Agent(init_state, behavior, dynamic, execution, shape,
                  params.AddChild("agent"))
    agent2 = Agent(init_state2, behavior, dynamic, execution, shape2,
                    params.AddChild("agent"))



if __name__ == '__main__':
  unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.


import unittest
import os
from bark.runtime.scenario.scenario_generation\
    import ScenarioGeneration

from bark.runtime.scenario.scenario_generation \
    import ConfigurableScenarioGeneration

from bark.runtime.scenario.scenario_generation.interaction_dataset_scenario_generation \
    import InteractionDatasetScenarioGeneration
from bark.runtime.scenario.scenario_generation.interaction_dataset_scenario_generation_full \
    import InteractionDatasetScenarioGenerationFull
from bark.runtime.commons import ParameterServer

from bark.core.geometry import *
import os


class ScenarioGenerationTests(unittest.TestCase):
    def test_configurable_scenario_generation_default_params(self):
        params = ParameterServer()
        mapfile = os.path.join(os.path.dirname(__file__),"data/city_highway_straight.xodr")
        params["Scenario"]["Generation"]["ConfigurableScenarioGeneration"]["MapFilename"] = mapfile
        scenario_generation = ConfigurableScenarioGeneration(
            num_scenarios=2, params=params)
        scenario_generation.dump_scenario_list("test.scenario")

        scenario_loader = ScenarioGeneration()
        scenario_loader.load_scenario_list("test.scenario")

        self.assertEqual(len(scenario_loader._scenario_list), 2)
        self.assertEqual(len(scenario_loader._scenario_list[0]._agent_list), len(
            scenario_generation._scenario_list[0]._agent_list))

        scenario = scenario_loader.get_scenario(idx=0)

        params.Save("default_params.json")

    def test_configurable_scenario_generation_sample_behavior_types(self):
        sink_source_dict = [{
            "SourceSink": [[5111.626, 5006.8305],  [5110.789, 5193.1725]],
            "Description": "left_lane",
            "ConfigAgentStatesGeometries": {"Type": "UniformVehicleDistribution", "LanePositions": [0]},
            "ConfigBehaviorModels": {"Type": "FixedBehaviorType", "ModelType": "BehaviorIDMClassic", "ModelParams":  {"BehaviorIDMClassic::MaxVelocity": 60.0}},
            "ConfigExecutionModels": {"Type": "FixedExecutionType"},
            "ConfigDynamicModels": {"Type": "FixedDynamicType"},
            "ConfigGoalDefinitions": {"Type": "FixedGoalTypes"},
            "ConfigControlledAgents": {"Type": "NoneControlled"},
            "AgentParams": {}
        },
            {
            "SourceSink": [[5111.626, 5006.8305],  [5110.789, 5193.1725]],
            "Description": "right_lane",
            "ConfigAgentStatesGeometries": {"Type": "UniformVehicleDistribution", "LanePositions": [1]},
            "ConfigBehaviorModels": {"Type": "SampleBehaviorType"},
            "ConfigExecutionModels": {"Type": "FixedExecutionType"},
            "ConfigDynamicModels": {"Type": "FixedDynamicType"},
            "ConfigGoalDefinitions": {"Type": "FixedGoalTypes"},
            "ConfigControlledAgents": {"Type": "RandomSingleAgent"},
            "AgentParams": {}
        }]
        params = ParameterServer()
        params["Scenario"]["Generation"]["ConfigurableScenarioGeneration"]["SinksSources"] = sink_source_dict
        params["Scenario"]["Generation"]["ConfigurableScenarioGeneration"]["MapFilename"] = os.path.join(os.path.dirname(__file__),"data/city_highway_straight.xodr")
        scenario_generation = ConfigurableScenarioGeneration(
            num_scenarios=2, params=params)
        scenario_generation.dump_scenario_list("test.scenario")

        params.Save("default_params_behavior_type_sampling.json")

    def test_find_overlaps_configurable_scenario_generation(self):
        shape = Polygon2d([0, 0, 0], [Point2d(-1, 0),
                                      Point2d(-1, 1),
                                      Point2d(1, 1),
                                      Point2d(1, 0)])

        agent_states1 = [[0, 1, 0, 0, 0], [0, 4, 0, 0, 0],
                         [0, 8, 0, 0, 0]]  # agents along x axis
        agent_geometries1 = [shape, shape, shape]

        agent_states2 = [[0, 4, -10, 0, 0], [0, 4, 0, 0, 0],
                         [0, 4, 20, 0, 0]]  # agents along y axis at x= 4
        agent_geometries2 = [shape, shape, shape]

        # some agents two colliding with other configs
        agent_states3 = [[0, 20, -20, 0, 0],
                         [0, 1, 0, 0, 0], [0, 4, 20, 0, 0]]
        agent_geometries3 = [shape, shape, shape]

        collected_sources_sinks_agent_states_geometries = [(agent_states1, agent_geometries1),
                                                           (agent_states2,
                                                            agent_geometries2),
                                                           (agent_states3, agent_geometries3)]

        overlaps = ConfigurableScenarioGeneration.find_overlaps_in_sources_sinks_agents(
            collected_sources_sinks_agent_states_geometries)

        self.assertTrue("0-1" in overlaps)

        collisions_01 = overlaps["0-1"]
        self.assertEqual(len(collisions_01), 1)

        # check source sink configs
        self.assertEqual(collisions_01[0][0][0], 0)
        self.assertEqual(collisions_01[0][1][0], 1)

        # check agent positions in list
        self.assertEqual(collisions_01[0][0][1], 1)
        self.assertEqual(collisions_01[0][1][1], 1)

        self.assertTrue("0-2" in overlaps)

        collisions_02 = overlaps["0-2"]
        self.assertEqual(len(collisions_02), 1)

        # check source sink configs
        self.assertEqual(collisions_02[0][0][0], 0)
        self.assertEqual(collisions_02[0][1][0], 2)

        # check agent positions in list
        self.assertEqual(collisions_02[0][0][1], 0)
        self.assertEqual(collisions_02[0][1][1], 1)

        collisions_03 = overlaps["1-2"]
        self.assertEqual(len(collisions_03), 1)

        # check source sink configs
        self.assertEqual(collisions_03[0][0][0], 1)
        self.assertEqual(collisions_03[0][1][0], 2)

        # check agent positions in list
        self.assertEqual(collisions_03[0][0][1], 2)
        self.assertEqual(collisions_03[0][1][1], 2)

    def test_dataset_scenario_generation_full(self):
        params = ParameterServer()

        map_filename =  os.path.join(os.path.dirname(__file__), "data/DR_DEU_Merging_MT_v01_shifted.xodr")
        track_filename =  os.path.join(os.path.dirname(__file__), "data/interaction_dataset_dummy_track.csv")

        params["Scenario"]["Generation"]["InteractionDatasetScenarioGenerationFull"]["MapFilename"] = map_filename
        params["Scenario"]["Generation"]["InteractionDatasetScenarioGenerationFull"]["TrackFilename"] = track_filename

        scenario_generation = InteractionDatasetScenarioGenerationFull(
            params=params, num_scenarios=3)
        # agent 1 is not part of the map, so it should only generate 2 scenarios

        assert(scenario_generation.get_num_scenarios() == 2)

    def test_dataset_scenario_generation(self):
        params = ParameterServer()


        map_filename = os.path.join(os.path.dirname(__file__), "data/DR_DEU_Merging_MT_v01_shifted.xodr")
        track_filename = os.path.join(os.path.dirname(__file__), "data/interaction_dataset_dummy_track.csv")

        params["Scenario"]["Generation"]["InteractionDatasetScenarioGeneration"]["MapFilename"] = map_filename
        params["Scenario"]["Generation"]["InteractionDatasetScenarioGeneration"]["TrackFilename"] = track_filename
        params["Scenario"]["Generation"]["InteractionDatasetScenarioGeneration"]["TrackIds"] = [
            1, 2]
        params["Scenario"]["Generation"]["InteractionDatasetScenarioGeneration"]["StartTs"] = 500
        params["Scenario"]["Generation"]["InteractionDatasetScenarioGeneration"]["EndTs"] = 1000
        params["Scenario"]["Generation"]["InteractionDatasetScenarioGeneration"]["EgoTrackId"] = 1

        scenario_generation = InteractionDatasetScenarioGeneration(
            params=params, num_scenarios=1)
        assert(scenario_generation.get_num_scenarios() == 1)


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import unittest
import os
from bark.core.world.map import Roadgraph
from bark.core.world.opendrive import XodrLane
from bark.runtime.commons import XodrParser


class RoadgraphTests(unittest.TestCase):
  def test_world(self):
    road_graph = Roadgraph()
    lane1 = XodrLane()
    lane2 = XodrLane()
    road_id = 0

    # add vertices and connect edges
    u = road_graph.AddLane(road_id, lane1)
    v = road_graph.AddLane(road_id, lane2)
    road_graph.AddInnerNeighbor(u, v)
    road_graph.AddOuterNeighbor(u, v)

    vertices = road_graph.GetVertices()
    for vertex_descr in vertices:
      vertex = road_graph.GetVertex(vertex_descr)
      print(vertex.lane_id)

    GetEdges = road_graph.GetEdges()
    for edge_descr in GetEdges:
      edge = road_graph.GetEdge(edge_descr)
      print(edge.edge_type)
  
  

if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import pickle 
import ray
import logging
import psutil
import inspect

import bark.core.world.evaluation
logging.getLogger().setLevel(logging.INFO)

from bark.runtime.scenario.scenario import Scenario
from bark.benchmark.benchmark_runner import BenchmarkRunner, BenchmarkResult, BenchmarkConfig

# implement a parallelized version of benchmark running based on ray

# custom serialization as cloudpickling fails
def serialize_benchmark_config(bc):
  return pickle.dumps(bc)

def deserialize_benchmark_config(bc):
  return pickle.loads(bc)

def serialize_scenario(sc):
  return pickle.dumps(sc)

def deserialize_scenario(sc):
  return pickle.loads(sc)


# actor class running on a single core
@ray.remote
class _BenchmarkRunnerActor(BenchmarkRunner):
    def __init__(self, evaluators, terminal_when, benchmark_configs, logger_name, log_eval_avg_every):
          super().__init__(evaluators=evaluators, 
                            terminal_when=terminal_when,
                            benchmark_configs=benchmark_configs,
                            logger_name=logger_name,
                            log_eval_avg_every=log_eval_avg_every)

# runner spawning actors and distributing benchmark configs
class BenchmarkRunnerMP(BenchmarkRunner):
    def __init__(self, benchmark_database=None,
               evaluators=None,
               terminal_when=None,
               behaviors=None,
               num_scenarios=None,
               benchmark_configs=None,
               log_eval_avg_every=None,
               num_cpus=None,
               memory_total=None):
        super().__init__(benchmark_database=benchmark_database,
                          evaluators=evaluators, terminal_when=terminal_when,
                          behaviors=behaviors, num_scenarios=num_scenarios,
                          benchmark_configs=benchmark_configs)
        num_cpus_available = psutil.cpu_count(logical=True)
        if num_cpus and num_cpus <= num_cpus_available:
          pass
        else:
          num_cpus = num_cpus_available
        
        mem = psutil.virtual_memory()
        memory_available = mem.available
        if memory_total and memory_total <= memory_available:
          pass
        else:
          memory_total = memory_available

        ray.init(num_cpus=num_cpus, memory=memory_total*0.3, object_store_memory=memory_total*0.7) # we split memory between workers (30%) and objects (70%)
        
        ray.register_custom_serializer(
          BenchmarkConfig, serializer=serialize_benchmark_config,
          deserializer=deserialize_benchmark_config)
        ray.register_custom_serializer(
          Scenario, serializer=serialize_scenario,
          deserializer=deserialize_scenario)
        self.benchmark_config_split = [self.benchmark_configs[i::num_cpus] for i in range(0, num_cpus)]
        self.actors = [_BenchmarkRunnerActor.remote(evaluators=evaluators,
                                                    terminal_when=terminal_when,
                                                    benchmark_configs=self.benchmark_config_split[i],
                                                    logger_name="BenchmarkingActor{}".format(i),
                                                    log_eval_avg_every=log_eval_avg_every) for i in range(num_cpus) ]

    def run(self, viewer = None, maintain_history = False):
        results_tmp = ray.get([actor.run.remote(viewer, maintain_history) for actor in self.actors])
        result_dict = []
        benchmark_configs = []
        histories = {}
        for result_tmp in results_tmp:
            result_dict.extend(result_tmp.get_result_dict())
            benchmark_configs.extend(result_tmp.get_benchmark_configs())
            histories.update(result_tmp.get_histories())
        return BenchmarkResult(result_dict, benchmark_configs, histories=histories)

    def __del__(self):
       ray.shutdown()


#!/usr/bin/env python

import matplotlib
import matplotlib.patches
import matplotlib.transforms
import numpy as np

from utils.dataset_types import Track, MotionState


def rotate_around_center(pts, center, yaw):
    return np.dot(pts - center, np.array([[np.cos(yaw), np.sin(yaw)], [-np.sin(yaw), np.cos(yaw)]])) + center


def polygon_xy_from_motionstate(ms, width, length):
    assert isinstance(ms, MotionState)
    lowleft = (ms.x - length / 2., ms.y - width / 2.)
    lowright = (ms.x + length / 2., ms.y - width / 2.)
    upright = (ms.x + length / 2., ms.y + width / 2.)
    upleft = (ms.x - length / 2., ms.y + width / 2.)
    return rotate_around_center(np.array([lowleft, lowright, upright, upleft]), np.array([ms.x, ms.y]), yaw=ms.psi_rad)


def update_objects_plot(timestamp, track_dict, patches_dict, text_dict, axes):
    for key, value in track_dict.items():
        assert isinstance(value, Track)
        if value.time_stamp_ms_first <= timestamp <= value.time_stamp_ms_last:
            # object is visible
            ms = value.motion_states[timestamp]
            assert isinstance(ms, MotionState)

            if key not in patches_dict:
                width = value.width
                length = value.length

                rect = matplotlib.patches.Polygon(polygon_xy_from_motionstate(ms, width, length), closed=True,
                                                  zorder=20)
                patches_dict[key] = rect
                axes.add_patch(rect)
                text_dict[key] = axes.text(ms.x, ms.y + 2, str(key), horizontalalignment='center', zorder=30)
            else:
                width = value.width
                length = value.length
                patches_dict[key].set_xy(polygon_xy_from_motionstate(ms, width, length))
                text_dict[key].set_position((ms.x, ms.y + 2))
        else:
            if key in patches_dict:
                patches_dict[key].remove()
                patches_dict.pop(key)
                text_dict[key].remove()
                text_dict.pop(key)



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.


import numpy as np
from bark.core.geometry import *
from bark.core.viewer import *
from bark.core.models.dynamic import *
from bark.runtime.viewer.viewer import BaseViewer
import threading
import time
from timeit import default_timer as timer


class ThreadedViewer(BaseViewer, threading.Thread):
    def __init__(self, renderer, params=None, **kwargs):
        threading.Thread.__init__(self)
        # Load 3d-Model and parameter
        super(ThreadedViewer, self).__init__(params=params)
        self.renderer = renderer
        self.world_queue_list = [] # objects of the world with 
        self.world_queue_world_times = []
        self.render_time = 0.0
        self.render_time_step = 0.03


    def _closest_world_queue_index(self):
        for idx, world_time in enumerate(reversed(self.world_queue_world_times)):
            if self.render_time >= world_time:
                return len(self.world_queue_world_times)- 1 - idx

    def drawWorld(self, world):
        self.world_queue_list.append(world.copy())
        self.world_queue_world_times.append(world.time)

    def reset(self):
        self.world_queue_list.clear()
        self.world_queue_world_times.clear()
        self.render_time = 0.0

        
    def run(self):
        while(True):
            if len(self.world_queue_list) == 0:
                continue
            current_world = self.world_queue_list[self._closest_world_queue_index()]
            executed_world = current_world.GetWorldAtTime(self.render_time)
            start = timer()
            self.renderer.drawWorld(executed_world)
            end = timer()
            self.render_time += self.render_time_step
            self.renderer.show(block=False)
            #time.sleep(self.render_time_step - (end-start)) # todo:measure rendering time






# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

from bark.runtime.scenario import Scenario
from bark.runtime.scenario.scenario_generation import ScenarioGeneration
from bark.runtime.commons import ModelJsonConversion
from bark.core.world.agent import *
from bark.core.models.behavior import *
from bark.core.world import *
from bark.core.world.goal_definition import *
from bark.core.world.map import *
from bark.core.models.dynamic import *
from bark.core.models.execution import *
from bark.core.geometry import *
from bark.core.geometry.standard_shapes import *
from bark.runtime.commons.parameters import ParameterServer
from bark.runtime.commons.xodr_parser import XodrParser
from bark.core.world.opendrive import *

import numpy as np
import math


class LaneCorridorConfig:
  """This class enables the configuration of a single LaneCorridor
     It assigns all models for the agents, determines their positions and more.
     Additionally, it can be chosen which agents should be controlled,
     what goal and model they should have.
  """
  def __init__(self,
               params=None,
               **kwargs):
    self._road_corridor = None
    self._params = params
    self._current_s = None
    self._lane_corridor = None

    # set these params
    self._road_ids = kwargs.pop("road_ids", None)
    self._lane_corridor_id = kwargs.pop("lane_corridor_id", None)
    self._s_min = kwargs.pop("s_min", 0.) 
    self._s_max = kwargs.pop("s_max", 60.)
    self._ds_min = kwargs.pop("ds_min", 10.)
    self._ds_max = kwargs.pop("ds_max", 20.)
    self._min_vel = kwargs.pop("min_vel", 8.)
    self._max_vel = kwargs.pop("max_vel", 10.)
    self._source_pos = kwargs.pop("source_pos", None)
    self._sink_pos = kwargs.pop("sink_pos", None)
    self._behavior_model = \
      kwargs.pop("behavior_model", BehaviorIDMClassic(self._params))
    self._controlled_behavior_model = \
      kwargs.pop("controlled_behavior_model", None)
    self._controlled_ids = kwargs.pop("controlled_ids", None)

  def InferRoadIdsAndLaneCorr(self, world):
    goal_polygon = Polygon2d([0, 0, 0],
                             [Point2d(-1,0),
                              Point2d(-1,1),
                              Point2d(1,1),
                              Point2d(1,0)])
    start_point = Point2d(self._source_pos[0], self._source_pos[1])
    end_point = Point2d(self._sink_pos[0], self._sink_pos[1])
    goal_polygon = goal_polygon.Translate(end_point)
    self._road_corridor = world.map.GenerateRoadCorridor(
      start_point, goal_polygon)
    self._road_ids = self._road_corridor.road_ids
    self._lane_corridor = self._road_corridor.GetCurrentLaneCorridor(
      start_point)
    
  def state(self, world):
    """Returns a state of the agent
    
    Arguments:
        world {bark.core.world}
    
    Returns:
        np.array -- time, x, y, theta, velocity
    """
    pose = self.position(world)
    if pose is None:
      return None
    velocity = self.velocity()
    return np.array([0, pose[0], pose[1], pose[2], velocity])

  def ds(self):
    """Increment for placing the agents
    
    Keyword Arguments:
        s_min {float} -- Min. lon. distance (default: {5.})
        s_max {float} -- Max. lon. distance (default: {10.})
    
    Returns:
        float -- delta s-value
    """
    return np.random.uniform(self._ds_min, self._ds_max)

  def position(self, world):
    """Using the defined LaneCorridor it finds positions for the agents
    
    Arguments:
        world {bark.core.world} -- BARK world
    
    Keyword Arguments:
        min_s {float} -- Min. lon. value (default: {0.})
        max_s {float} -- Max. lon. value (default: {100.})
    
    Returns:
        tuple -- (x, y, theta)
    """
    if self._road_corridor == None:
      world.map.GenerateRoadCorridor(
        self._road_ids, XodrDrivingDirection.forward)
      self._road_corridor = world.map.GetRoadCorridor(
        self._road_ids, XodrDrivingDirection.forward)
    if self._road_corridor is None:
      return None
    if self._lane_corridor is not None:
      lane_corr = self._lane_corridor
    else:
      lane_corr = self._road_corridor.lane_corridors[self._lane_corridor_id]
    if lane_corr is None:
      return None
    centerline = lane_corr.center_line
    if self._current_s == None:
      self._current_s = self._s_min
    xy_point =  GetPointAtS(centerline, self._current_s)
    angle = GetTangentAngleAtS(centerline, self._current_s)
    if self._current_s > self._s_max:
      return None
    self._current_s += self.ds()
    return (xy_point.x(), xy_point.y(), angle)

  def velocity(self):
    return np.random.uniform(low=self._min_vel, high=self._max_vel)

  def behavior_model(self, world):
    """Returns behavior model
    """
    return self._behavior_model

  @property
  def execution_model(self):
    """Returns exec. model
    """
    return ExecutionModelInterpolate(self._params)

  @property
  def dynamic_model(self):
    """Returns dyn. model
    """
    return SingleTrackModel(self._params)

  @property
  def shape(self):
    """Returns shape
    """
    return Polygon2d([0, 0, 0], [Point2d(-2, -1),
                      Point2d(2, -1),
                      Point2d(2, 1),
                      Point2d(-2, 1),
                      Point2d(-2, -1)])

  def goal(self, world):
    """Returns goal def.
    """
    # TODO: by default should be based on agent's pos
    road_corr = world.map.GetRoadCorridor(
      self._road_ids, XodrDrivingDirection.forward)
    if self._lane_corridor:
      lane_corr = self._lane_corridor
    else:
      lane_corr = self._road_corridor.lane_corridors[self._lane_corridor_id]
    # TODO: check
    return GoalDefinitionStateLimitsFrenet(lane_corr.center_line,
                                           (0.2, 0.2),
                                           (0.1, 0.1),
                                           (10., 15.))

  def controlled_ids(self, agent_list):
    """Returns an ID-List of controlled agents
    """
    if self._controlled_ids is None:
      return []
    random_int = [agent_list[np.random.randint(0, len(agent_list))]]
    return random_int

  def controlled_goal(self, world):
    """Goal for the controlled agent
    
    Arguments:
        world {bark.core.world} -- BARK world
    
    Returns:
        GoalDefinition -- Goal for the controlled agent
    """
    return self.goal(world)

  def controlled_behavior_model(self, world):
    """Behavior model for controlled agent
    
    Returns:
        BehaviorModel -- BARK behavior model
    """
    if self._controlled_behavior_model is None:
      return self.behavior_model(world)

  def reset(self):
    """Resets the LaneCorridorConfig
    """
    self._current_s = None


class ConfigWithEase(ScenarioGeneration):
  """Configure your scenarios with ease
     &copy; Patrick Hart
  """
  def __init__(self,
               num_scenarios,
               map_file_name=None,
               params=None,
               random_seed=None,
               lane_corridor_configs=None):
    self._map_file_name = map_file_name
    self._lane_corridor_configs = lane_corridor_configs or []
    self._map_interface = None
    super(ConfigWithEase, self).__init__(params, num_scenarios)
    self.initialize_params(params)

  def create_scenarios(self, params, num_scenarios):
    """see baseclass
    """
    scenario_list = []
    for scenario_idx in range(0, num_scenarios):
      scenario = self.create_single_scenario()
      scenario_list.append(scenario)
    return scenario_list

  def create_single_scenario(self):
    """Creates one scenario using the defined LaneCorridorConfig
    
    Returns:
        Scenario -- Returns a BARK scenario
    """
    scenario = Scenario(map_file_name=self._map_file_name,
                        json_params=self._params.ConvertToDict())
    # as we always use the same world, we can create the MapIntf. once
    if self._map_interface is None:
      scenario.CreateMapInterface(self._map_file_name)
    else:
      scenario.map_interface = self._map_interface
    self._map_interface = scenario.map_interface
    world = scenario.GetWorldState()
    map_interface = world.map
    # fill agent list of the BARK world and set agents that are controlled
    scenario._agent_list = []
    scenario._eval_agent_ids = []
    for lc_config in self._lane_corridor_configs:
      agent_state = True
      lc_agents = []
      if lc_config._source_pos is not None and lc_config._sink_pos is not None:
        lc_config.InferRoadIdsAndLaneCorr(world)
      while agent_state is not None:
        agent_state = lc_config.state(world)
        if agent_state is not None:
          agent_behavior = lc_config.behavior_model(world)
          agent_dyn = lc_config.dynamic_model
          agent_exec = lc_config.execution_model
          agent_polygon = lc_config.shape
          agent_params = self._params.AddChild("agent")
          agent_goal = lc_config.goal(world)
          new_agent = Agent(
            agent_state, 
            agent_behavior, 
            agent_dyn,
            agent_exec, 
            agent_polygon,
            agent_params,
            agent_goal,
            map_interface)
          new_agent.road_corridor = lc_config._road_corridor
          lc_agents.append(new_agent)
        # set the road corridor

      # handle controlled agents
      controlled_agent_ids = []
      for controlled_agent in lc_config.controlled_ids(lc_agents):
        controlled_agent.goal_definition = lc_config.controlled_goal(world)
        controlled_agent.behavior_model = \
          lc_config.controlled_behavior_model(world)
        controlled_agent_ids.append(controlled_agent.id)
      scenario._eval_agent_ids.extend(controlled_agent_ids)
      scenario._agent_list.extend(lc_agents)
      lc_config.reset()
    return scenario


# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import numpy as np
import time
import os
from bark.runtime.commons.parameters import ParameterServer
from bark.runtime.viewer.matplotlib_viewer import MPViewer
from bark.runtime.viewer.video_renderer import VideoRenderer
from bark.runtime.scenario.scenario_generation.config_with_ease import \
  LaneCorridorConfig, ConfigWithEase
from bark.runtime.runtime import Runtime
from bark.runtime.viewer.panda3d_easy import Panda3dViewer
from bark.core.models.behavior import *


# parameters
param_server = ParameterServer()
param_server["BehaviorLaneChangeRuleBased"]["MinVehicleRearDistance"] = 4.
param_server["BehaviorLaneChangeRuleBased"]["MinVehicleFrontDistance"] = 2.
param_server["BehaviorLaneChangeRuleBased"]["TimeKeepingGap"] = 0.

# custom lane configuration that sets a different behavior model
# and sets the desired speed for the behavior
class HighwayLaneCorridorConfig(LaneCorridorConfig):
  def __init__(self, params=None, **kwargs):
    super().__init__(params=params, **kwargs)
    self._count = 0
  
  def velocity(self):
    return np.random.uniform(2., 10.)

  def behavior_model(self, world):
    # agent_params = ParameterServer()
    params = self._params.AddChild("BehaviorLaneChangeRuleBased"+str(self._count))
    params["BehaviorIDMClassic"]["DesiredVelocity"] = \
      np.random.uniform(5., 10.)
    behavior_model = BehaviorLaneChangeRuleBased(params)
    self._count += 1
    return behavior_model


# configure both lanes of the highway. the right lane has one controlled agent
left_lane = HighwayLaneCorridorConfig(params=param_server,
                                      road_ids=[16],
                                      lane_corridor_id=0)
right_lane = HighwayLaneCorridorConfig(params=param_server,
                                       road_ids=[16],
                                       lane_corridor_id=1,
                                       controlled_ids=True)

# create 5 scenarios
scenarios = \
  ConfigWithEase(
    num_scenarios=5,
    map_file_name=os.path.join(os.path.dirname(__file__),"../runtime/tests/data/city_highway_straight.xodr"),
    random_seed=0,
    params=param_server,
    lane_corridor_configs=[left_lane, right_lane])

# viewer
viewer = MPViewer(params=param_server,
                  x_range=[-75, 75],
                  y_range=[-75, 75],
                  follow_agent_id=True)

sim_step_time = param_server["simulation"]["step_time",
                                          "Step-time used in simulation",
                                          0.05]
sim_real_time_factor = param_server["simulation"][
  "real_time_factor",
  "execution in real-time or faster",
  0.5]

# viewer = VideoRenderer(renderer=viewer,
#                        world_step_time=sim_step_time,
#                        fig_path="/home/hart/Dokumente/2020/bark/video")

# gym like interface
env = Runtime(step_time=0.2,
              viewer=viewer,
              scenario_generator=scenarios,
              render=True)
      


# run 3 scenarios
for episode in range(0, 1):
  env.reset()
  # step each scenario 20 times
  for step in range(0, 70):
    env.step()
    time.sleep(sim_step_time/sim_real_time_factor)

# viewer.export_video(filename="/home/hart/Dokumente/2020/bark/video/video", remove_image_dir=False)



from .scenario_generation import ScenarioGeneration
from .deterministic import  DeterministicScenarioGeneration
from .drone_challenge import DroneChallengeScenarioGeneration
from .config_with_ease import LaneCorridorConfig
from .uniform_vehicle_distribution import UniformVehicleDistribution
from .deterministic_drone_challenge import  DeterministicDroneChallengeGeneration
from .configurable_scenario_generation import ConfigurableScenarioGeneration
from .interaction_dataset_scenario_generation import InteractionDatasetScenarioGeneration
from .interaction_dataset_scenario_generation_full import InteractionDatasetScenarioGenerationFull


__all__ = ["DeterministicScenarioGeneration",
          "DroneChallengeScenarioGeneration",
          "LaneCorridorConfig",
          "ScenarioGeneration",
          "UniformVehicleDistribution",
          "DeterministicDroneChallengeGeneration",
          "ConfigurableScenarioGeneration",
          "InteractionDatasetScenarioGeneration",
          "InteractionDatasetScenarioGenerationFull"]



#!/usr/bin/env python

import matplotlib
import matplotlib.axes
import matplotlib.pyplot as plt

import xml.etree.ElementTree as xml
import pyproj
import math

from utils import dict_utils


class Point:
    def __init__(self):
        self.x = None
        self.y = None


class LL2XYProjector:
    def __init__(self, lat_origin, lon_origin):
        self.lat_origin = lat_origin
        self.lon_origin = lon_origin
        self.zone = math.floor((lon_origin+180.)/6)+1  # works for most tiles, and for all in the dataset
        self.p = pyproj.Proj(proj='utm', ellps='WGS84', zone=self.zone, datum='WGS84')
        [self.x_origin, self.y_origin] = self.p(lon_origin, lat_origin)

    def latlon2xy(self, lat, lon):
        [x, y] = self.p(lon, lat)
        return [x-self.x_origin, y-self.y_origin]


def get_type(element):
    for tag in element.findall("tag"):
        if tag.get("k") == "type":
            return tag.get("v")
    return None


def get_subtype(element):
    for tag in element.findall("tag"):
        if tag.get("k") == "subtype":
            return tag.get("v")
    return None


def get_x_y_lists(element, point_dict):
    x_list = list()
    y_list = list()
    for nd in element.findall("nd"):
        pt_id = int(nd.get("ref"))
        point = point_dict[pt_id]
        x_list.append(point.x)
        y_list.append(point.y)
    return x_list, y_list


def set_visible_area(point_dict, axes):
    min_x = 10e9
    min_y = 10e9
    max_x = -10e9
    max_y = -10e9

    for id, point in dict_utils.get_item_iterator(point_dict):
        min_x = min(point.x, min_x)
        min_y = min(point.y, min_y)
        max_x = max(point.x, max_x)
        max_y = max(point.y, max_y)

    axes.set_aspect('equal', adjustable='box')
    axes.set_xlim([min_x - 10, max_x + 10])
    axes.set_ylim([min_y - 10, max_y + 10])


def draw_map_without_lanelet(filename, axes, lat_origin, lon_origin):

    assert isinstance(axes, matplotlib.axes.Axes)

    axes.set_aspect('equal', adjustable='box')
    axes.patch.set_facecolor('lightgrey')

    projector = LL2XYProjector(lat_origin, lon_origin)

    e = xml.parse(filename).getroot()

    point_dict = dict()
    for node in e.findall("node"):
        point = Point()
        point.x, point.y = projector.latlon2xy(float(node.get('lat')), float(node.get('lon')))
        point_dict[int(node.get('id'))] = point

    set_visible_area(point_dict, axes)

    unknown_linestring_types = list()

    for way in e.findall('way'):
        way_type = get_type(way)
        if way_type is None:
            raise RuntimeError("Linestring type must be specified")
        elif way_type == "curbstone":
            type_dict = dict(color="black", linewidth=1, zorder=10)
        elif way_type == "line_thin":
            way_subtype = get_subtype(way)
            if way_subtype == "dashed":
                type_dict = dict(color="white", linewidth=1, zorder=10, dashes=[10, 10])
            else:
                type_dict = dict(color="white", linewidth=1, zorder=10)
        elif way_type == "line_thick":
            way_subtype = get_subtype(way)
            if way_subtype == "dashed":
                type_dict = dict(color="white", linewidth=2, zorder=10, dashes=[10, 10])
            else:
                type_dict = dict(color="white", linewidth=2, zorder=10)
        elif way_type == "pedestrian_marking":
            type_dict = dict(color="white", linewidth=1, zorder=10, dashes=[5, 10])
        elif way_type == "bike_marking":
            type_dict = dict(color="white", linewidth=1, zorder=10, dashes=[5, 10])
        elif way_type == "stop_line":
            type_dict = dict(color="white", linewidth=3, zorder=10)
        elif way_type == "virtual":
            type_dict = dict(color="blue", linewidth=1, zorder=10, dashes=[2, 5])
        elif way_type == "road_border":
            type_dict = dict(color="black", linewidth=1, zorder=10)
        elif way_type == "guard_rail":
            type_dict = dict(color="black", linewidth=1, zorder=10)
        elif way_type == "traffic_sign":
            continue
        else:
            if way_type not in unknown_linestring_types:
                unknown_linestring_types.append(way_type)
            continue

        x_list, y_list = get_x_y_lists(way, point_dict)
        plt.plot(x_list, y_list, **type_dict)

    if len(unknown_linestring_types) != 0:
        print("Found the following unknown types, did not plot them: " + str(unknown_linestring_types))



# Copyright (c) 2019 fortiss GmbH
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT

import os
import logging
import random
import pickle
import time


from bark.runtime.scenario.scenario_generation.scenario_generation import ScenarioGeneration
from bark.runtime.scenario.scenario_generation.uniform_vehicle_distribution import UniformVehicleDistribution
from bark.runtime.scenario.scenario_generation.deterministic import DeterministicScenarioGeneration
from bark.runtime.scenario.scenario_generation.configurable_scenario_generation import ConfigurableScenarioGeneration
from bark.runtime.scenario.scenario_generation.interaction_dataset_scenario_generation import InteractionDatasetScenarioGeneration
from bark.runtime.scenario.scenario_generation.interaction_dataset_scenario_generation_full import InteractionDatasetScenarioGenerationFull

from bark.runtime.commons.parameters import ParameterServer
from bark.runtime.viewer.matplotlib_viewer import MPViewer


import bark.runtime.scenario.scenario_generation 

FILE_EXTENSION_SCENARIO_SET = "bark_scenarios"

# The ScenarioSetSerializer dumps, loads and
# tests all scenarios sets specified in one param file


class ScenarioSetSerializer:
    def __init__(self, params):
        self._params = params
        self._scenario_generator_name = self._params["Scenario"]["Generation"]["GeneratorName"]
        self._generator_seed =  self._params["Scenario"]["Generation"]["GeneratorSeed"]
        self._num_scenarios =  self._params["Scenario"]["Generation"]["NumScenarios"]
        self._set_name =   self._params["Scenario"]["Generation"]["SetName"]
        self._simulation_step_time = self._params["Simulation"]["StepTime"]
        self._scenario_generator = None
        self._last_serialized_filename = None
    
    @staticmethod
    def scenario_file_name(set_name, num_scenarios, seed):
        return "{}_scenarios{}_seed{}.{}".format(set_name, num_scenarios, seed, FILE_EXTENSION_SCENARIO_SET)

    @staticmethod
    def scenario_set_info_filename(set_name):
        return "set_info_{}".format(set_name)

    @staticmethod
    def scenario_set_info_fileprefix():
        return "set_info"

    def dump(self, dir):
        self._dump(dir, self._scenario_generator_name, self._num_scenarios, self._generator_seed)

    def _dump(self, dir, generator, num_scenarios, seed, **kwargs):
        self._scenario_generator = eval("{}( \
                num_scenarios={}, params=self._params, random_seed={})".format(self._scenario_generator_name,
                                                                    self._num_scenarios,
                                                                    self._generator_seed))
        filename = os.path.join(dir, ScenarioSetSerializer.scenario_file_name(
            self._set_name, self._num_scenarios, self._generator_seed
        ))
        if not os.path.exists(os.path.dirname(filename)):
            os.makedirs(os.path.dirname(filename))
        self._scenario_generator.dump_scenario_list(filename)
        self._last_serialized_filename = filename

        param_file_name = os.path.join(dir, os.path.basename(self._params.param_filename))
        info_dict = {"GeneratorName": generator, "SetName": self._set_name, "NumScenarios": num_scenarios,
                     "Seed": seed, "Serialized": filename, "Params": param_file_name, **kwargs}

        info_filename = os.path.join(dir, ScenarioSetSerializer.scenario_set_info_filename(self._set_name))
        with open(info_filename, "wb") as f:
            pickle.dump(info_dict , f)

    def load(self, filename=None):
        if not filename and self._last_serialized_filename:
            filename = self._last_serialized_filename
        self._scenario_generator = ScenarioGeneration()
        self._scenario_generator.load_scenario_list(filename=filename)

    def test(self, num_scenarios, num_steps, visualize_test, viewer=None, test_scenario_idxs = None):
        if not self._scenario_generator:
            logging.error("No scenario generator initialized for testing")
            return

        logging.info("Testing {} with seed {} from generator {}".format(
            self._set_name, self._generator_seed, self._scenario_generator_name
        ))

        results = []
        if not test_scenario_idxs:
            test_scenario_idxs = random.sample(list(range(0, num_scenarios)), num_scenarios)
        for scenario_idx in test_scenario_idxs: # run all scenarios
            result = self._test_scenario(scenario_idx, num_steps, visualize_test, viewer)
            results.append(result)

        failed = not all(results)
        failed_scenario_idx = [idx for idx, rst in enumerate(results) if rst == False]
        if failed:
            logging.error("Failed scenario indexes {} during testing of {} with seed {} and generator {}".format(
            failed_scenario_idx, self._set_name, self._generator_seed, self._scenario_generator_name))
            return False
        return True


    def _test_scenario(self, scenario_idx, num_steps, visualize, viewer):
            logging.info("Running scenario {} of {} in set {}".format(scenario_idx,
                                                                self._scenario_generator.num_scenarios,
                                                                self._set_name))
            try:
                scenario = self._scenario_generator.get_scenario(scenario_idx)
            except Exception as e:
                logging.error("Deserialization failed with {}.".format(e))
                return False
            try:
                world_state = scenario.get_world_state()
            except Exception as e:
                logging.error("Get world state failed with {}.".format(e))
                return False

            if visualize:
                if not viewer:
                  viewer = MPViewer(
                  params=ParameterServer(),
                  x_range=[5060, 5160],
                  y_range=[5070,5150],
                  use_world_bounds=True)

            sim_step_time = 0.2
            sim_real_time_factor = 1

            try:
                for _ in range(0, num_steps): # run a few steps for each scenario
                  if visualize:
                        info_text = "SetName: {} | ScenarioIdx: {}".format(self._set_name, scenario_idx)
                        viewer.drawText(position = (0.5, 1.05), text=info_text)
                        viewer.drawWorld(world_state, scenario._eval_agent_ids, scenario_idx=scenario_idx)
                        viewer.show(block=False)
                        time.sleep(sim_step_time/sim_real_time_factor)
                        viewer.clear()
                  world_state.Step(sim_step_time)
                return True
            except Exception as e:
                logging.error("Simulation failed with {}.".format(e))
                return False





# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import pygame as pg
import numpy as np
from bark.core.geometry import *
from bark.core.viewer import *
from bark.core.world.opendrive import *
from bark.core.models.dynamic import *
from bark.runtime.viewer.viewer import BaseViewer


class PygameViewer(BaseViewer):
    def __init__(self, params=None, **kwargs):
        super(PygameViewer, self).__init__(params=params, **kwargs)

        self.screen_dims = kwargs.pop("screen_dims", np.array([600, 600]))
        self.screen_width = self.screen_dims[0]
        self.screen_height = self.screen_dims[1]
        self.screen_map_ratio = None
        self.source_dest = None
        self.screen_surface = pg.Surface(
            self.screen_dims, pg.DOUBLEBUF | pg.HWSURFACE)

        self.camera_view_size = None

        self.map_surf = None
        self.map_surf_size = None

        # NOTE: Optimizied way to support alpha value in pygame
        # https://stackoverflow.com/questions/6339057/draw-a-transparent-rectangle-in-pygame
        self.alpha_surf = dict()
        self.background_color = (255, 255, 255)

        pg.font.init()
        try:
            self.screen = pg.display.set_mode(
                self.screen_dims, pg.DOUBLEBUF | pg.HWSURFACE)
            pg.display.set_caption("Bark")
            self.clear()
        except pg.error:
            self.screen = None
            print("No available video device")

    def drawMap(self, map):
        """ Draw Opendrive map on a map surface and initialize camera surface properties according to the map size"""
        if self.map_surf is None:
            # Find boundary to create fixed size pygame surf
            self.map_min_bound = np.full(2, np.inf)
            self.map_max_bound = np.full(2, -np.inf)
            lanes_np = []
            lanes_dashed = []

            for _, road in map.GetRoads().items():
                for lane_section in road.lane_sections:
                    for _, lane in lane_section.GetLanes().items():
                        lane_np = lane.line.ToArray()
                        lanes_np.append(lane_np)
                        lanes_dashed.append(lane.road_mark.type == XodrRoadMarkType.broken
                                            or lane.road_mark.type == XodrRoadMarkType.none)

                        self.map_min_bound = np.minimum(
                            self.map_min_bound, np.amin(lane_np, axis=0))
                        self.map_max_bound = np.maximum(
                            self.map_max_bound, np.amax(lane_np, axis=0))

                self.camera_view_size = np.array([np.diff(self.dynamic_world_x_range)[
                    0], np.diff(self.dynamic_world_y_range)[0]])

                if self.use_world_bounds:
                    # Scale to the map size
                    self.screen_map_ratio = min(
                        self.screen_dims / self.camera_view_size)
                    self.map_surf_size = self.screen_dims
                else:
                    # Scale larger to have detailed visualization
                    self.screen_map_ratio = max([self.screen_width / (np.diff(self.world_x_range)[0]),
                                                 self.screen_height / (np.diff(self.world_y_range)[0])])
                    self.map_surf_size = (
                        self.map_max_bound - self.map_min_bound) * self.screen_map_ratio

            self.map_surf = pg.Surface(self.map_surf_size)
            self.map_surf.fill(self.background_color)

            for lane_np, lane_dashed in zip(lanes_np, lanes_dashed):
                if lane_dashed:
                    self.drawDashedLines(self.map_surf, self.getColor(
                        self.color_lane_boundaries), self.mapToSurfaceCoordinates(lane_np), 3)
                else:
                    pg.draw.lines(self.map_surf, self.getColor(
                        self.color_lane_boundaries), False, self.mapToSurfaceCoordinates(lane_np), 3)

        if self.use_world_bounds:
            camera_coordinate = np.array([0, 0])
            camera_view_range = self.screen_dims

            map_x_range = self.map_max_bound[0] - self.map_min_bound[0]
            map_y_range = self.map_max_bound[1] - self.map_min_bound[1]

            # Calculate source destination to blit map to the center of screen
            if map_x_range > map_y_range:
                self.source_dest = (
                    0, int(map_x_range - map_y_range) / 2 * self.screen_map_ratio)
            elif map_x_range < map_y_range:
                self.source_dest = (
                    int(map_y_range - map_x_range) / 2 * self.screen_map_ratio, 0)
            else:
                self.source_dest = (0, 0)

        else:
            # Project the coordinate of top left corner of dynamic world window
            camera_coordinate = self.mapToSurfaceCoordinates(
                np.array([self.dynamic_world_x_range[0], self.dynamic_world_y_range[1]]))
            camera_view_range = self.camera_view_size * self.screen_map_ratio
            self.source_dest = (0, 0)

        self.screen_surface.blit(self.map_surf, self.source_dest,
                                 np.around((camera_coordinate, camera_view_range)))

    def drawPoint2d(self, point2d, color, alpha=1.0):
        transformed_points = self.pointsToCameraCoordinate(point2d)
        s = self.createTransparentSurace(
            self.screen_dims, self.background_color, alpha)
        pg.draw.circle(s, self.getColor(color),
                       transformed_points, 1, 0)

    def drawLine2d(self, line2d, color="blue", alpha=1.0,
                   dashed=False, zorder=10, linewidth=1):
        transformed_lines = self.pointsToCameraCoordinate(line2d)
        s = self.createTransparentSurace(
            self.screen_dims, self.background_color, alpha)
        if dashed:
            self.drawDashedLines(
                s, self.getColor(color), transformed_lines, 3)

    def drawPolygon2d(self, polygon, color="blue", alpha=1.0, facecolor=None):
        transformed_points = self.pointsToCameraCoordinate(polygon)
        s = self.createTransparentSurace(
            self.screen_dims, self.background_color, alpha)
        pg.draw.polygon(s, self.getColor(color), transformed_points)

    def drawTrajectory(self, trajectory, color):
        if len(trajectory) < 1:
            return
        point_list = []
        for state in trajectory:
            point_list.append([state[round(StateDefinition.X_POSITION)],
                               state[round(StateDefinition.Y_POSITION)]])

        pg.draw.lines(self.screen_surface, self.getColor(color), False,
                      self.pointsToCameraCoordinate(point_list), 5)

    def drawText(self, position, text, **kwargs):
        font = pg.font.get_default_font()
        font_size = kwargs.pop("fontsize", 18)
        color = kwargs.pop("color", (0, 0, 0))
        background_color = kwargs.pop(
            "background_color", self.background_color)
        text_surf = pg.font.SysFont(font, font_size).render(
            text, True, color, background_color)
        self.screen_surface.blit(
            text_surf, (position[0] * self.screen_width, (1 - position[1]) * self.screen_height))

    def getColor(self, color):
        if isinstance(color, str):
            return {
                "white": pg.Color(255, 255, 255),
                "red": pg.Color(255, 0, 0),
                "blue": pg.Color(0, 0, 255),
                "magenta": pg.Color(100, 255, 100),
                "brown": pg.Color(150, 50, 50),
                "black": pg.Color(0, 0, 0)
            }.get(color, pg.Color(0, 0, 0))
        else:
            return pg.Color(int(color[0] * 255),
                            int(color[1] * 255),
                            int(color[2] * 255))

    def drawWorld(self, world, eval_agent_ids=None, show=True):
        super(PygameViewer, self).drawWorld(world, eval_agent_ids)

        if self.alpha_surf:
            for s in self.alpha_surf.values():
                self.screen_surface.blit(s, (0, 0))

        if show:
            self.show()

    def show(self, block=True):
        if self.screen is None:
            return

        self.screen.blit(self.screen_surface, (0, 0))

        pg.display.flip()
        pg.event.get()  # call necessary for visbility of pygame viewer on macos

    def clear(self):
        # Clear each surface, could occupy lot of memory if using many
        # different alpha values
        if self.alpha_surf:
            for s in self.alpha_surf.values():
                s.fill(self.background_color)

        self.screen.fill(self.background_color)
        self.screen_surface.fill(self.background_color)

    def getColorFromMap(self, float_color):
        # TODO
        return (1.0, 0, 0)

    def get_aspect_ratio(self):
        return 1

    """
        Steps to project coordinates:
        1. Mirror by y-axis
        2. Translate y-axis by the max y-coordinate in the map
        3. Translate x-axis by the max x-coordinate in the map
        4. Scaling by screen_map_ratio
    """

    def mapToSurfaceCoordinates(self, points):
        """ Project Opendrive map coordinates into pygame surface coordinates"""
        return (points - np.array([self.map_min_bound[0], self.map_max_bound[1]])
                ) * np.array([1, -1]) * self.screen_map_ratio

    def pointsToCameraCoordinate(self, points):
        """ Project Opendrive map coordinates into camera surface coordinates"""
        if isinstance(points, list):
            points = np.array(points)
        elif not isinstance(points, np.ndarray):
            points = points.ToArray()

        if self.use_world_bounds:
            return np.array([0, self.screen_height]) + (points - np.array([self.dynamic_world_x_range[0], self.dynamic_world_y_range[0]])) * np.array([1, -1]) \
                * self.screen_map_ratio
        else:
            return np.array([0, self.screen_height]) + (points - np.array([self.dynamic_world_x_range[0], self.dynamic_world_y_range[0]])) * np.array([1, -1]) \
                / self.camera_view_size * self.screen_dims

    def createTransparentSurace(self, dims, background_color, alpha):
        alpha = round(float(alpha), 2)
        if alpha not in self.alpha_surf:
            s = pg.Surface(dims, pg.DOUBLEBUF | pg.HWSURFACE)
            s.fill(background_color)
            s.set_colorkey(background_color)
            s.set_alpha(int(alpha * 255))
            self.alpha_surf[alpha] = s
            return s
        else:
            return self.alpha_surf[alpha]

    def drawDashedLines(self, surf, color, points, width, dashed_length=5):
        for i in range(len(points) - 1):
            origin = points[i]
            target = points[i + 1]
            diff = target - origin
            length = np.linalg.norm(diff)
            slope = diff / length

            for j in np.arange(0, length / dashed_length, 2):
                start = origin + slope * j * dashed_length
                end = start + slope * dashed_length
                pg.draw.line(surf, color, start, end, width)



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import unittest
import numpy as np
import time
from bark.runtime.commons import ParameterServer
from bark.runtime.viewer import MPViewer
from bark.runtime.commons import XodrParser
from bark.core.models.behavior import BehaviorConstantVelocity, BehaviorMobil, BehaviorIDMLaneTracking
from bark.core.models.execution import ExecutionModelInterpolate
from bark.core.models.dynamic import SingleTrackModel
from bark.core.world import World, MakeTestWorldHighway
from bark.core.world.goal_definition import GoalDefinitionPolygon
from bark.core.world.agent import Agent
from bark.core.world.map import MapInterface, Roadgraph
from bark.core.geometry.standard_shapes import CarLimousine, CarRectangle
from bark.core.geometry import Point2d, Polygon2d
from bark.core.world.evaluation import EvaluatorDrivableArea
from bark.core.world.opendrive import OpenDriveMap, XodrRoad, PlanView, \
    MakeXodrMapOneRoadTwoLanes, XodrLaneSection, XodrLane


class WorldTests(unittest.TestCase):
    """This test should create and obstacle, map and agent insert it into the world.
      FM, the world should be stepped several times in order to verify the functionality.
    """

    def test_world(self):
        # create agent
        params = ParameterServer()
        behavior = BehaviorConstantVelocity(params)
        execution = ExecutionModelInterpolate(params)
        dynamic = SingleTrackModel(params)
        shape = Polygon2d([1.25, 1, 0], [
            Point2d(0, 0),
            Point2d(0, 2),
            Point2d(4, 2),
            Point2d(4, 0),
            Point2d(0, 0)
        ])
        init_state = np.array([0, 0, 0, 0, 5])
        agent = Agent(init_state, behavior, dynamic, execution, shape,
                      params.AddChild("agent"))
        road_map = OpenDriveMap()
        newXodrRoad = XodrRoad()
        newXodrRoad.id = 1
        newXodrRoad.name = "Autobahn A9"
        newPlanView = PlanView()
        newPlanView.AddLine(Point2d(0, 0), 1.57079632679, 10)
        newXodrRoad.plan_view = newPlanView
        line = newXodrRoad.plan_view.GetReferenceLine().ToArray()
        p = Point2d(line[-1][0], line[-1][1])
        newXodrRoad.plan_view.AddSpiral(p, 1.57079632679, 50.0, 0.0, 0.3, 0.4)
        line = newXodrRoad.plan_view.GetReferenceLine()
        lane_section = XodrLaneSection(0)
        lane = XodrLane()
        lane.line = line
        lane_section.AddLane(lane)
        newXodrRoad.AddLaneSection(lane_section)
        road_map.AddRoad(newXodrRoad)

        r = Roadgraph()
        map_interface = MapInterface()
        map_interface.SetOpenDriveMap(road_map)
        map_interface.SetRoadgraph(r)
        world = World(params)
        world.AddAgent(agent)

    def test_highway(self):
        world = MakeTestWorldHighway()

    def test_evaluator_drivable_area(self):
        # World Definition
        params = ParameterServer()
        world = World(params)

        # Model Definitions
        behavior_model = BehaviorConstantVelocity(params)
        execution_model = ExecutionModelInterpolate(params)
        dynamic_model = SingleTrackModel(params)

        # Map Definition
        map_interface = MapInterface()
        xodr_map = MakeXodrMapOneRoadTwoLanes()
        map_interface.SetOpenDriveMap(xodr_map)
        world.SetMap(map_interface)
        #open_drive_map = world.map.GetOpenDriveMap()

        #agent_2d_shape = CarLimousine()
        agent_2d_shape = Polygon2d(
            [1.25, 1, 0], [Point2d(-1, -1), Point2d(-1, 1), Point2d(3, 1), Point2d(3, -1)])
        init_state = np.array([0, 3, -1.75, 0, 5])
        agent_params = params.AddChild("agent1")
        goal_polygon = Polygon2d(
            [1, 1, 0], [Point2d(0, 0), Point2d(0, 2), Point2d(2, 2), Point2d(2, 0)])
        goal_polygon = goal_polygon.Translate(Point2d(50, -2))

        agent = Agent(init_state,
                      behavior_model,
                      dynamic_model,
                      execution_model,
                      agent_2d_shape,
                      agent_params,
                      GoalDefinitionPolygon(goal_polygon),  # goal_lane_id
                      map_interface)
        world.AddAgent(agent)

        evaluator = EvaluatorDrivableArea()
        world.AddEvaluator("drivable_area", evaluator)

        info = world.Evaluate()
        self.assertFalse(info["drivable_area"])

        viewer = MPViewer(params=params,
                          use_world_bounds=True)

        # Draw map
        viewer.drawGoalDefinition(goal_polygon, color=(1,0,0), alpha=0.5, facecolor= (1,0,0))
        viewer.drawWorld(world)
        viewer.drawRoadCorridor(agent.road_corridor)
        viewer.show(block=False)

    def test_lane_change(self):
        # World Definition
        params = ParameterServer()
        world = World(params)

        # Model Definitions
        behavior_model = BehaviorMobil(params)
        execution_model = ExecutionModelInterpolate(params)
        dynamic_model = SingleTrackModel(params)

        behavior_model2 = BehaviorIDMLaneTracking(params)
        execution_model2 = ExecutionModelInterpolate(params)
        dynamic_model2 = SingleTrackModel(params)

        # Map Definition
        map_interface = MapInterface()
        xodr_map = MakeXodrMapOneRoadTwoLanes()
        map_interface.SetOpenDriveMap(xodr_map)
        world.SetMap(map_interface)

        #agent_2d_shape = CarLimousine()
        agent_2d_shape = CarRectangle()
        init_state = np.array([0, 3, -1.75, 0, 5])
        agent_params = params.AddChild("agent1")
        goal_polygon = Polygon2d(
            [1, 1, 0], [Point2d(0, 0), Point2d(0, 2), Point2d(2, 2), Point2d(2, 0)])
        goal_polygon = goal_polygon.Translate(Point2d(50, -2))

        agent = Agent(init_state, behavior_model, dynamic_model, execution_model,
                      agent_2d_shape, agent_params, GoalDefinitionPolygon(goal_polygon), map_interface)
        world.AddAgent(agent)

        init_state2 = np.array([0, 15, -1.75, 0, 2])
        agent2 = Agent(init_state2, behavior_model2, dynamic_model2, execution_model2,
                       agent_2d_shape, agent_params, GoalDefinitionPolygon(goal_polygon), map_interface)
        world.AddAgent(agent2)

        # viewer
        viewer = MPViewer(params=params, use_world_bounds=True)

        # World Simulation
        sim_step_time = params["simulation"]["step_time",
                                             "Step-time in simulation", 0.05]
        sim_real_time_factor = params["simulation"]["real_time_factor",
                                                    "execution in real-time or faster", 100]

        # Draw map
        for _ in range(0, 10):
            viewer.clear()
            world.Step(sim_step_time)
            viewer.drawWorld(world)
            viewer.show(block=False)
            time.sleep(sim_step_time/sim_real_time_factor)


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.


import unittest
import os
import numpy as np
from bark.runtime.scenario.scenario_generation.deterministic \
  import DeterministicScenarioGeneration
from bark.runtime.scenario.scenario_generation.scenario_generation \
  import ScenarioGeneration

from bark.core.world.goal_definition import GoalDefinition, GoalDefinitionPolygon
from bark.core.geometry import *
from bark.core.world import World
from bark.runtime.commons.parameters import ParameterServer
from bark.runtime.runtime import Runtime
from bark.runtime.viewer.matplotlib_viewer import MPViewer
from bark.core.models.behavior import BehaviorModel, BehaviorDynamicModel
from bark.core.models.dynamic import SingleTrackModel


class PythonBehaviorModelWrapper(BehaviorModel):
  """Dummy Python behavior model
  """
  def __init__(self,
               dynamic_model = None,
               params = None):
    # BehaviorDynamicModel.__init__(self, dynamic_model, params)
    BehaviorModel.__init__(self, params)
    self._dynamic_model = dynamic_model
    self._params = params

  def Plan(self, delta_time, world):
    super(PythonBehaviorModelWrapper, self).ActionToBehavior(
      np.array([2., 1.], dtype=np.float32))
    # print(super(PythonBehaviorModelWrapper, self).GetAction())
    trajectory = np.array([[0., 0., 0., 0., 0.],
                           [0., 0., 0., 0., 0.]], dtype=np.float32)
    super(PythonBehaviorModelWrapper, self).SetLastTrajectory(trajectory)
    return trajectory

  def Clone(self):
    return self


class PythonBehaviorModelWrapperInheritance(BehaviorModel):
  """Dummy Python behavior model
  """
  def __init__(self,
               params = None):
    BehaviorModel.__init__(
      self, params)
    self._dynamic_behavior_model = BehaviorDynamicModel(params)
  
  def Plan(self, delta_time, world):
    self._dynamic_behavior_model.ActionToBehavior(
      np.array([2., 1.], dtype=np.float32))
    trajectory = self._dynamic_behavior_model.Plan(delta_time, world)
    super(PythonBehaviorModelWrapperInheritance, self).SetLastTrajectory(trajectory)
    return trajectory

  def Clone(self):
    return self


class PyBehaviorModelTests(unittest.TestCase):
  def test_python_model(self):
    param_server = ParameterServer(
      filename= os.path.join(os.path.dirname(__file__),"../../runtime/tests/data/deterministic_scenario.json"))
    param_server
    
    mapfile = os.path.join(os.path.dirname(__file__),"../../runtime/tests/data/city_highway_straight.xodr")
    param_server["Scenario"]["Generation"]["DeterministicScenarioGeneration"]["MapFilename"] = mapfile
    scenario_generation = DeterministicScenarioGeneration(num_scenarios=3,
                                                          random_seed=0,
                                                          params=param_server)
    viewer = MPViewer(params=param_server,
                      follow_agent_id=False,
                      use_world_bounds=True)
    scenario, idx = scenario_generation.get_next_scenario()
    world = scenario.GetWorldState()
    single_track_model = SingleTrackModel(param_server)
    behavior_model = PythonBehaviorModelWrapper(
      single_track_model, param_server)
    world.GetAgent(0).behavior_model = behavior_model
    world.GetAgent(0).behavior_model.SetLastAction(
      np.array([1., 1.], dtype=np.float32))
    world.Step(0.2)

  def test_python_model_inheritance(self):
    param_server = ParameterServer(
      filename= os.path.join(os.path.dirname(__file__),"../../runtime/tests/data/deterministic_scenario.json"))
    mapfile = os.path.join(os.path.dirname(__file__),"../../runtime/tests/data/city_highway_straight.xodr")
    param_server["Scenario"]["Generation"]["DeterministicScenarioGeneration"]["MapFilename"] = mapfile
    
    scenario_generation = DeterministicScenarioGeneration(num_scenarios=3,
                                                          random_seed=0,
                                                          params=param_server)
    viewer = MPViewer(params=param_server,
                      follow_agent_id=False,
                      use_world_bounds=True)
    scenario, idx = scenario_generation.get_next_scenario()
    world = scenario.GetWorldState()
    behavior_model = PythonBehaviorModelWrapperInheritance(param_server)
    
    world.GetAgent(0).behavior_model = behavior_model
    world.GetAgent(0).behavior_model.SetLastAction(
      np.array([1., 1.], dtype=np.float32))
    world.Step(0.2)


if __name__ == '__main__':
  unittest.main()


# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import numpy as np
from bark.runtime.commons.parameters import ParameterServer
from bark.runtime.commons.xodr_parser import XodrParser
from bark.core.models.execution import ExecutionModelInterpolate
from bark.core.models.dynamic import SingleTrackModel, StateDefinition
from bark.core.world import World, MakeTestWorldHighway
from bark.core.world.goal_definition import GoalDefinitionPolygon, GoalDefinitionStateLimitsFrenet
from bark.core.world.agent import Agent
from bark.core.world.map import MapInterface, Roadgraph
from bark.core.geometry.standard_shapes import CarLimousine, CarRectangle
from bark.core.geometry import Point2d, Polygon2d, Line2d
from bark.core.world.evaluation import EvaluatorDrivableArea
from bark.core.world.opendrive import OpenDriveMap, XodrRoad, PlanView, \
    MakeXodrMapOneRoadTwoLanes, XodrLaneSection, XodrLane
from bark.core.models.behavior import * 
import os


def make_initial_world(primitives):
    # must be within examples params folder
    params = ParameterServer()
    world = World(params)

    # Define two behavior models
    behavior_model = BehaviorMPContinuousActions(params)
    primitive_mapping = {}
    for prim in primitives:
      idx = behavior_model.AddMotionPrimitive(np.array(prim)) # adding action
      primitive_mapping[idx] = prim

    behavior_model.ActionToBehavior(0) # setting initial action

    execution_model = ExecutionModelInterpolate(params)
    dynamic_model = SingleTrackModel(params)

    behavior_model2 = BehaviorConstantVelocity(params)
    execution_model2 = ExecutionModelInterpolate(params)
    dynamic_model2 = SingleTrackModel(params)

    # Define the map interface and load a testing map
    map_interface = MapInterface()
    xodr_map = MakeXodrMapOneRoadTwoLanes()
    map_interface.SetOpenDriveMap(xodr_map)
    world.SetMap(map_interface)

    # Define the agent shapes
    agent_2d_shape = CarRectangle()
    init_state = np.array([0, 3, -5.25, 0, 20])

    # Define the goal definition for agents
    center_line = Line2d()
    center_line.AddPoint(Point2d(0.0, -1.75))
    center_line.AddPoint(Point2d(100.0, -1.75))

    max_lateral_dist = (0.4, 0.5)
    max_orientation_diff = (0.08, 0.1)
    velocity_range = (5.0, 20.0)
    goal_definition = GoalDefinitionStateLimitsFrenet(center_line,
                                                      max_lateral_dist, max_orientation_diff,
                                                      velocity_range)

    # define two agents with the different behavior models
    agent_params = params.AddChild("agent1")
    agent = Agent(init_state, behavior_model, dynamic_model, execution_model,
                  agent_2d_shape, agent_params, goal_definition, map_interface)
    world.AddAgent(agent)

    init_state2 = np.array([0, 25, -5.25, 0, 15])
    agent2 = Agent(init_state2, behavior_model2, dynamic_model2, execution_model2,
                   agent_2d_shape, agent_params, goal_definition, map_interface)
    world.AddAgent(agent2)

    return world

def get_ego_agent(world):
  return world.agents[0]

def apply_action_to_ego_agent(world, idx):
  get_ego_agent(world).behavior_model.ActionToBehavior(idx)



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.
import matplotlib
from matplotlib.patches import Polygon
from matplotlib import cm
import matplotlib.pyplot as plt

from bark.core.viewer import *
from bark.core.geometry import *
from bark.core.models.dynamic import StateDefinition

from bark.runtime.viewer.viewer import BaseViewer


class MPViewer(BaseViewer):
    # we do not need an init function as pybind11 implements it
    def __init__(self, params=None, **kwargs):
        super(MPViewer, self).__init__(params=params, **kwargs)
        if 'axis' in kwargs:
          self.axes = kwargs.pop("axis")
        else:
          self.axes = plt.subplots()[1]
          # removes whitespace
          plt.subplots_adjust(bottom=0.0, left=0.0, right=1.0, top=1)

    def drawPoint2d(self, point2d, color, alpha):
        self.axes.plot(
            point2d.x(),
            point2d.y(),
            color=self.getColor(color),
            alpha=alpha,
            marker='x')

    def drawLine2d(self, line2d, color='blue', alpha=1.0, dashed=False, zorder=1, linewidth=1):
        lineStyle_string = (0, (5, 10)) if dashed else 'solid'
        line2d_np = line2d.ToArray()
        self.axes.plot(
            line2d_np[:, 0],
            line2d_np[:, 1], 
            lineStyle=lineStyle_string,
            color=self.getColor(color),
            alpha=alpha,
            zorder=zorder,
            linewidth=linewidth)

    def drawPolygon2d(self, polygon, color, alpha, facecolor=None, linewidth=1):
        points = polygon.ToArray()
        polygon_draw = matplotlib.patches.Polygon(
            points,
            True,
            facecolor=self.getColor(facecolor),
            edgecolor=self.getColor(color),
            alpha=alpha,
            linewidth=linewidth)
        t_start = self.axes.transData
        polygon_draw.set_transform(t_start)
        self.axes.add_patch(polygon_draw)
        center = polygon.center
        self.axes.plot(center[0], center[1], color=self.getColor(color))

    def drawTrajectory(self, trajectory, color):
        if len(trajectory) > 0:
            self.axes.plot(
                trajectory[:, int(StateDefinition.X_POSITION)],
                trajectory[:, int(StateDefinition.Y_POSITION)],
                color=self.getColor(color))

    def drawText(self, position, text, coordinate="axes", **kwargs):
        if coordinate=="axes":
            self.axes.text(position[0], position[1], text, horizontalalignment='center',
             verticalalignment='top', transform=self.axes.transAxes, **kwargs)
        else:
            self.axes.text(position[0], position[1], text, horizontalalignment='center',
             verticalalignment='top', **kwargs)

    def getColor(self, color):
        if isinstance(color, Viewer.Color):
            return {
                Viewer.Color.White: 'w',
                Viewer.Color.Red: 'red',
                Viewer.Color.Blue: 'darkblue',
                Viewer.Color.Magenta: 'darkgreen',
                Viewer.Color.Brown: 'sienna'
            }.get(color, 'k')
        else:
            return color

    def get_aspect_ratio(self):
      # ratio is width / height
      [w, h] = self._get_ax_size()
      return (w/h)

    def _get_ax_size(self):
      fig = self.axes.get_figure()
      scale = fig.dpi_scale_trans.inverted()
      bbox = self.axes.get_window_extent().transformed(scale)
      width, height = bbox.width, bbox.height
      width *= fig.dpi
      height *= fig.dpi
      return width, height

    def drawWorld(self, world, eval_agent_ids=None, filename=None, scenario_idx=None, debug_text=True):
        # self.clear()
        plt.axis('off')
        super(MPViewer, self).drawWorld(world, eval_agent_ids, filename, scenario_idx, debug_text)
        self._set_visualization_options()
        self.show()
        if filename:
            self.axes.get_figure().savefig(filename)

    def saveFig(self, filename):
      self.axes.get_figure().savefig(filename)

    def show(self, block=False):
        plt.draw()
        if block:
            plt.show(block=True)
        else:
            plt.pause(0.001)

    def _set_visualization_options(self):
        # x and y limits
        self.axes.set_xlim(self.dynamic_world_x_range[0], self.dynamic_world_x_range[1])
        self.axes.set_ylim(self.dynamic_world_y_range[0], self.dynamic_world_y_range[1])

        self.axes.get_xaxis().set_visible(False)
        self.axes.get_yaxis().set_visible(False)

    def clear(self):
        self.axes.cla()
      
    
    def getColorFromMap(self, float_color):
        return cm.Accent(float_color)



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

from bark.core.world.agent import Agent
from bark.core.world import World
from bark.core.world.map import MapInterface
from bark.runtime.commons.parameters import ParameterServer
from bark.runtime.commons.xodr_parser import XodrParser
import copy
from pathlib import Path


class Scenario:
  def __init__(self,
               agent_list=None,
               eval_agent_ids=None,
               map_file_name=None,
               json_params=None,
               map_interface=None):
    self._agent_list = agent_list or []
    self._eval_agent_ids = eval_agent_ids or []
    self._map_file_name = map_file_name
    self._json_params = json_params
    self._map_interface = map_interface

  @property
  def map_file_name(self):
    return self._map_file_name
  
  @property
  def json_params(self):
    return self._json_params

  @property
  def eval_agent_ids(self):
    return self._eval_agent_ids

  @property
  def map_interface(self):
    return self._map_interface

  @map_interface.setter
  def map_interface(self, map_interface):
    self._map_interface = map_interface
  
  def GetWorldState(self):
    """get initial world state of scenario to start simulation from here
    
    Returns:
        [bark.core.world.World]
    """
    return self._build_world_state()

  def copy(self):
    return Scenario(agent_list=copy.deepcopy(self._agent_list),
                    eval_agent_ids=self._eval_agent_ids.copy(),
                    map_file_name=self._map_file_name,
                    json_params=self._json_params.copy(),
                    map_interface=self._map_interface)

  def _build_world_state(self):
    param_server = ParameterServer(json=self._json_params)
    world = World(param_server)
    if self._map_interface is None:
      self.CreateMapInterface(self._map_file_name)
      world.SetMap(self._map_interface)
    else:
      world.SetMap(self._map_interface)
    for agent in self._agent_list:
      agent.GenerateRoadCorridor(self._map_interface)
      world.AddAgent(agent)
    world.UpdateAgentRTree()
    return world

  def __getstate__(self):
    odict = self.__dict__.copy()
    del odict['_map_interface']
    return odict

  def __setstate__(self, sdict):
    sdict['_map_interface'] = None
    self.__dict__.update(sdict)

  # TODO(@hart): should be a commons function
  def CreateMapInterface(self, map_file_name):
    map_file_load_test = Path(map_file_name)
    if map_file_load_test.is_file():
      xodr_parser = XodrParser(map_file_name)
    else:
      objects_found = sorted(Path().rglob(map_file_name))
      if len(objects_found) == 0:
        raise ValueError("No Map found")
      elif len(objects_found) > 1:
        raise ValueError("Multiple Maps found")
      else:
        xodr_parser = XodrParser(objects_found[0].as_posix())

    map_interface = MapInterface()
    map_interface.SetOpenDriveMap(xodr_parser.map)
    self._map_interface = map_interface








#!/usr/bin/env python

import argparse

from utils import dataset_reader
from utils import dataset_types
from utils import dict_utils

if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument("filename", type=str, help="Path to the track file", nargs="?")
    args = parser.parse_args()

    if args.filename is None:
        raise IOError("You must specify a filename. Type --help for help.")

    track_dictionary = dataset_reader.read_tracks(args.filename)

    print("Found " + str(len(track_dictionary)) + " object tracks.")

    track = dict_utils.get_value_list(track_dictionary)[0]
    assert isinstance(track, dataset_types.Track)
    print("Track with id " + str(track.track_id) + " lasts from ts " + str(track.time_stamp_ms_first) + \
          " to ts " + str(track.time_stamp_ms_last) + ", so " +
          str((track.time_stamp_ms_last - track.time_stamp_ms_first) / 1000.) + " seconds.")

    motion_state = track.motion_states[track.time_stamp_ms_first]
    assert isinstance(motion_state, dataset_types.MotionState)
    print("Its initial motion state is " + str(motion_state))




# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import numpy as np
import logging
from bark.core.viewer import Viewer
from bark.core.geometry import *
from bark.core.models.dynamic import *
from bark.core.world.opendrive import *
from bark.core.world.goal_definition import *
from bark.runtime.commons.parameters import ParameterServer
import math

logger = logging.getLogger()

class BaseViewer(Viewer):
    def __init__(self, params=None, **kwargs):
        if(params is None):
           params = ParameterServer()
        Viewer.__init__(self)
        # color parameters
        # agents
        self.color_other_agents_line = params["Visualization"]["Agents"]["Color"]["Other"]["Lines", "Color of other agents", (0.7,0.7,0.7)]
        self.color_other_agents_face = params["Visualization"]["Agents"]["Color"]["Other"]["Face", "Color of other agents", (0.7,0.7,0.7)]
        self.color_eval_agents_line = params["Visualization"]["Agents"]["Color"]["Controlled"]["Lines", "Color of controlled, evaluated agents", (0.,.27,.58)]
        self.color_eval_agents_face = params["Visualization"]["Agents"]["Color"]["Controlled"]["Face", "Color of controlled, evaluated agents", (.49, .63, .83)]
        self.use_colormap_for_other_agents = params["Visualization"]["Agents"]["Color"]["UseColormapForOtherAgents", "Flag to enable color map for other agents", False]
        self.alpha_eval_agent = params["Visualization"]["Agents"]["Alpha"]["Controlled", "Alpha of evalagents", 1.]
        self.alpha_other_agents = params["Visualization"]["Agents"]["Alpha"]["Other", "Alpha of other agents", 1]
        self.route_color =  params["Visualization"]["Agents"]["ColorRoute", "Color of agents routes", (0.2,0.2,0.2)]
        self.draw_route = params["Visualization"]["Agents"]["DrawRoute", "Draw Route of each agent", False]
        self.draw_agent_id = params["Visualization"]["Agents"]["DrawAgentId", "Draw id of each agent", True]
        self.draw_eval_goals = params["Visualization"]["Agents"]["DrawEvalGoals", "Draw Route of eval agent goals", True]
        self.eval_goal_color = params["Visualization"]["Agents"]["EvalGoalColor", "Color of eval agent goals", (.49, .63, .83)]
        self.draw_history = params["Visualization"]["Agents"]["DrawHistory", "Draw history with alpha trace for each agent", False]
        # map
        self.color_lane_boundaries = params["Visualization"]["Map"]["XodrLanes"]["Boundaries"]["Color", "Color of agents except ego vehicle", (0.7,0.7,0.7)]
        self.alpha_lane_boundaries = params["Visualization"]["Map"]["XodrLanes"]["Boundaries"]["Alpha", "Color of agents except ego vehicle", 1.0]
        self.plane_color = params["Visualization"]["Map"]["Plane"]["Color", "Color of the background plane", (1, 1, 1, 1)]
        self.plane_alpha = params["Visualization"]["Map"]["Plane"]["Alpha", "Alpha of the background plane", 1.0]
        self.map_linewidth = params["Visualization"]["Map"]["XodrLanes"]["Boundaries"]["Linewidth", "Linewidth of linestrings", 1.0]

        self.parameters = params

        self.use_world_bounds = kwargs.pop("use_world_bounds", False)
        self.follow_agent_id = kwargs.pop("follow_agent_id", None)

        self.center = kwargs.pop("center", np.array([0, 0]))

        self.world_x_range = kwargs.pop("x_range", np.array([-40, 40]))
        self.world_y_range = kwargs.pop("y_range", np.array([-40, 40]))

        self.enforce_x_length = kwargs.pop("enforce_x_length", True)
        self.enforce_y_length = kwargs.pop("enforce_y_length", False)
        self.x_length = kwargs.pop("x_length", np.sum(np.absolute(self.world_x_range)))
        self.y_length = kwargs.pop("y_length", np.sum(np.absolute(self.world_y_range)))

        self.dynamic_world_x_range = self.world_x_range.copy()
        self.dynamic_world_y_range = self.world_y_range.copy()

    def reset():
        pass

    def get_aspect_ratio(self):
        pass

    def _get_draw_eval_agent_ids(self, world, eval_agent_ids=None, ):
        if self.follow_agent_id is not None:
            if isinstance(self.follow_agent_id, bool) and \
                     eval_agent_ids is not None and \
                     len(eval_agent_ids) == 1:
                draw_eval_agent_id = eval_agent_ids[0]
            else:
                draw_eval_agent_id = self.follow_agent_id

            if draw_eval_agent_id in world.agents:
                return draw_eval_agent_id

        return None

    def _update_world_view_range(self, world, eval_agent_ids=None):
        draw_eval_agent_id = self._get_draw_eval_agent_ids(world, eval_agent_ids)

        if draw_eval_agent_id != None:
            follow_agent = world.agents[draw_eval_agent_id]
            state = follow_agent.state
            pose = np.zeros(3)
            pose[0] = state[int(StateDefinition.X_POSITION)]
            pose[1] = state[int(StateDefinition.Y_POSITION)]
            pose[2] = state[int(StateDefinition.THETA_POSITION)]

            center = [pose[0],  pose[1]]
            self._update_world_dynamic_range(center)
        else:
          if self.use_world_bounds:
              bb = world.bounding_box
              self.dynamic_world_x_range = [bb[0].x(), bb[1].x()]
              self.dynamic_world_y_range = [bb[0].y(), bb[1].y()]

              diffx = abs(self.dynamic_world_x_range[1] - self.dynamic_world_x_range[0])
              diffy = abs(self.dynamic_world_y_range[1] - self.dynamic_world_y_range[0])

              # enforce that in both dimensions  the same range is covered
              if diffx > diffy:
                  self.dynamic_world_y_range[0] -= (diffx - diffy)/2
                  self.dynamic_world_y_range[1] += (diffx - diffy)/2
              else:
                  self.dynamic_world_x_range[0] -= (diffy - diffx)/2
                  self.dynamic_world_x_range[1] += (diffy - diffx)/2
          else:
              center = self.center
              # self._update_world_dynamic_range(center)


    def _update_world_dynamic_range(self, center):
        aspect_ratio = self.get_aspect_ratio()
        if self.enforce_x_length:
            self.dynamic_world_x_range = [-self.x_length/2 + center[0], self.x_length/2 + center[0]]
            self.dynamic_world_y_range = [-self.x_length/2/aspect_ratio + center[1], self.x_length/2/aspect_ratio + center[1]]

        if self.enforce_y_length:
            self.dynamic_world_x_range = [-self.y_length/2*aspect_ratio + center[0], self.y_length/2*aspect_ratio + center[0]]
            self.dynamic_world_y_range = [-self.y_length/2 + center[1], self.y_length/2 + center[1]]

    def drawPoint2d(self, point2d, color, alpha):
        pass

    def drawLine2d(self, line2d, color, alpha, line_style=None, zorder=10):
        pass

    def drawPolygon2d(self, polygon, color, alpha, facecolor=None):
        pass

    def drawTrajectory(self, trajectory, color):
        pass

    def drawObstacle(self, obstacle):
        pass

    def drawText(self, position, text, **kwargs):
        pass

    def getColor(self, color):
        pass

    def show(self,block=False):
        pass

    def clear(self):
        pass

    def drawAgents(self, world):
        for _, agent in world.agents.items():
            self.drawAgent(agent)

    def drawHistory(self, agent, color, alpha, facecolor):
        shape = agent.shape
        if isinstance(shape, Polygon2d):
            history = agent.history
            lh = len(history)
            for idx, state_action in enumerate(history):
                state = state_action[0]
                pose = np.zeros(3)
                # pybind creates column based vectors, initialization maybe row-based -> we consider both
                pose[0] = state[int(StateDefinition.X_POSITION)]
                pose[1] = state[int(StateDefinition.Y_POSITION)]
                pose[2] = state[int(StateDefinition.THETA_POSITION)]
                transformed_polygon = shape.Transform(pose)
                alpha=1-0.8*(lh-idx)/4
                alpha = 0 if alpha<0 else alpha
                self.drawPolygon2d(transformed_polygon, color, alpha, facecolor) # fade to 0.2 after 10 steps
    
    def drawGoalDefinition(self, goal_definition, color, alpha, facecolor):
        if isinstance(goal_definition, GoalDefinitionPolygon):
            self.drawPolygon2d(goal_definition.goal_shape, color, alpha, facecolor)
        elif isinstance(goal_definition, GoalDefinitionStateLimits):
            self.drawPolygon2d(goal_definition.xy_limits, color, alpha, facecolor)
        elif isinstance(goal_definition, GoalDefinitionStateLimitsFrenet):
            self.drawPolygon2d(goal_definition.goal_shape, color, alpha, facecolor)
        elif isinstance(goal_definition, GoalDefinitionSequential):
            prev_center = np.array([])
            for idx, goal_def in enumerate(goal_definition.sequential_goals):
                self.drawGoalDefinition(goal_def, color, alpha, facecolor)
                goal_pos = None
                if isinstance(goal_def, GoalDefinitionPolygon):
                    goal_pos = goal_def.goal_shape.center
                elif isinstance(goal_def, GoalDefinitionStateLimits):
                    goal_pos = goal_def.xy_limits.center
                # self.drawText(position=goal_pos, text="Goal{}".format(idx), coordinate="world")
                if prev_center.any():
                    line = Line2d()
                    line.AddPoint(Point2d(prev_center[0], prev_center[1]))
                    line.AddPoint(Point2d(goal_pos[0], goal_pos[1]))
                    self.drawLine2d(line, color, alpha=0.9)
                prev_center = goal_pos

    def drawWorld(self, world, eval_agent_ids=None, filename=None, scenario_idx=None, debug_text=True):
        # self.clear()
        self._update_world_view_range(world, eval_agent_ids)
        if world.map:
            self.drawMap(world.map.GetOpenDriveMap())

        # draw agent goals
        for agent_id, agent in world.agents.items():
            if eval_agent_ids and self.draw_eval_goals and agent.goal_definition and \
                  agent_id in eval_agent_ids:
                color_line = self.eval_goal_color
                color_face = self.eval_goal_color
                alpha = .5
                self.drawGoalDefinition(agent.goal_definition, color_line, alpha, color_face)

        num_agents = len(world.agents.items())
        for i, (agent_id, agent) in enumerate(world.agents.items()):
            color = "blue"
            alpha = 1.0
            if eval_agent_ids and agent.id in eval_agent_ids:
                color_line = self.color_eval_agents_line
                color_face = self.color_eval_agents_face
                alpha = self.alpha_eval_agent
            else:
                alpha = self.alpha_other_agents
                if self.use_colormap_for_other_agents:
                  color_line = self.getColorFromMap(float(i) / float(num_agents))
                  color_face = self.getColorFromMap(float(i) / float(num_agents))
                else:
                  color_line = self.color_other_agents_line
                  color_face = self.color_other_agents_face
            self.drawAgent(agent, color_line, alpha, color_face)
            if self.draw_history:
                self.drawHistory(agent, color_line, alpha, color_face)
        if debug_text:
          self.drawText(position=(0.1, 0.9), text="Scenario: {}".format(scenario_idx), fontsize=14)
          self.drawText(position=(0.1, 0.95), text="Time: {:.2f}".format(world.time), fontsize=14)
      

    def drawMap(self, map):
        # draw the boundary of each lane
        for _, road in map.GetRoads().items():
            self.drawXodrRoad(road, self.color_lane_boundaries)

    def drawXodrRoad(self, road, color=None):
        for lane_section in road.lane_sections:
          self.drawXodrLaneSection(lane_section, color)
    
    def drawXodrLaneSection(self, lane_section, color=None):
      for _, lane in lane_section.GetLanes().items():
        self.drawXodrLane(lane, color)
        
    def drawXodrLane(self, lane, color=None):
      if color is None:
        self.color_lane_boundaries

      dashed = False
      # center line is type none and is drawn as broken
      if lane.road_mark.type == XodrRoadMarkType.broken or lane.road_mark.type == XodrRoadMarkType.none: 
        dashed = True
      self.drawLine2d(lane.line, color, self.alpha_lane_boundaries, dashed, zorder=2, linewidth=self.map_linewidth)

    def drawAgent(self, agent, color, alpha, facecolor):
        shape = agent.shape
        if isinstance(shape, Polygon2d):
            pose = np.zeros(3)
            # pybind creates column based vectors, initialization maybe row-based -> we consider both
            state = agent.state
            pose[0] = state[int(StateDefinition.X_POSITION)]
            pose[1] = state[int(StateDefinition.Y_POSITION)]
            pose[2] = state[int(StateDefinition.THETA_POSITION)]
            transformed_polygon = shape.Transform(pose)

            centerx = (shape.front_dist - 0.5*(shape.front_dist+shape.rear_dist)) * math.cos(pose[2]) + pose[0]
            centery = (shape.front_dist - 0.5*(shape.front_dist+shape.rear_dist))* math.sin(pose[2]) + pose[1]

            if self.draw_agent_id:
              self.drawText(position=(centerx, centery), rotation=180.0*(1.0+pose[2]/math.pi), text="{}".format(agent.id),\
                 coordinate="not axes", ha='center', va="center", multialignment="center", size="smaller")
            
            self.drawPolygon2d(transformed_polygon, color, alpha, facecolor)
        else:
            raise NotImplementedError("Shape drawing not implemented.")

    def drawLaneCorridor(self, lane_corridor, color="blue"):
      self.drawPolygon2d(lane_corridor.polygon, color=color, alpha=.5)

    def drawRoadCorridor(self, road_corridor, color="blue"):
      # TODO(@hart): use agent specific coloring
      self.drawPolygon2d(road_corridor.polygon, color=color, alpha=.2)
      for lane_corridor in road_corridor.lane_corridors:
        self.drawLaneCorridor(lane_corridor)

    def Reset(self):
      pass


# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

from bark.runtime.scenario import Scenario
from bark.runtime.scenario.scenario_generation import ScenarioGeneration
from bark.runtime.commons import ModelJsonConversion
from bark.core.world.agent import *
from bark.core.models.behavior import *
from bark.core.world import *
from bark.core.world.goal_definition import GoalDefinition, GoalDefinitionPolygon, GoalDefinitionSequential
from bark.core.world.map import *
from bark.core.models.dynamic import *
from bark.core.models.execution import *
from bark.core.geometry import *
from bark.core.geometry.standard_shapes import *
from bark.runtime.commons import ParameterServer
from bark.runtime.commons import XodrParser

import numpy as np
import math


class DroneChallengeScenarioGeneration(ScenarioGeneration):
  def __init__(self, num_scenarios, params=None, random_seed=None):
    super(DroneChallengeScenarioGeneration, self).__init__(params,
                                                          num_scenarios)
    self.initialize_params(params)

  def initialize_params(self, params):
    self._local_params = \
      self._params["Scenario"]["Generation"]["DeterministicScenarioGeneration"]
    
    self.goal_frame_center = self._local_params["goal_frame"]["center_pose", "Center pose of the goal frames", [0,0,0]]
    self.goal_frame_points = self._local_params["goal_frame"]["polygon_points",
                                                 "Points of the goal frame polygon",
                                                 [[-0.2,1],[0.2, 1],[0.2,-1],[-0.2,-1],[-0.2,1]]]
    self.goal_frame_poses = self._local_params["goal_poses", "A list with x,y, theta specyfing the sequence of the goal frames",
                    [[5,10, 3.14/2+0.01], [10,15, 3.14/4],[20,10, 3.14/2] ,[15,2, 3.14/2-0.7]]]

    json_converter = ModelJsonConversion()
    _agent_params = json_converter.agent_to_json(self.default_drone_model())
    if not isinstance(_agent_params, dict):
        _agent_params = _agent_params.ConvertToDict()

    self.drone_params = self._local_params["drones", "list of dictionaries with drone definitions", [{"drone_model": _agent_params}]]
    self.ego_agent_id = self._local_params["ego_agent_id","ID of the ego-agent", 0]

  def create_scenarios(self, params, num_scenarios):
    """ 
        see baseclass
    """
    scenario_list = []
    for scenario_idx in range(0, num_scenarios):
      scenario = self.create_single_scenario()     
      scenario_list.append(scenario)
    return scenario_list


  def _build_sequential_goal_definition(self):
    goal_list = []
    for goal_pose in self.goal_frame_poses:
        goal_polygon = Polygon2d(self.goal_frame_center,
                               np.array(self.goal_frame_points))
        goal_polygon = goal_polygon.Transform(goal_pose)
        goal_definition = GoalDefinitionPolygon(goal_polygon)
        goal_list.append(goal_definition)
    
    return GoalDefinitionSequential(goal_list)

  def create_single_scenario(self):
    scenario = Scenario(map_file_name=None,
                        json_params=self._params.ConvertToDict())
    scenario._agent_list = []
    for agent_json in self.drone_params:
      agent_json["drone_model"]["map_interface"] = None
      agent_json["drone_model"]["goal_definition"] = self._build_sequential_goal_definition()
      json_converter = ModelJsonConversion()
      agent = json_converter.agent_from_json(agent_json["drone_model"],
                                                   param_server=self._local_params)
      agent.SetAgentId(agent_json["drone_model"]["id"])
      scenario._agent_list.append(agent)

    scenario._eval_agent_ids = [self._local_params["EgoAgentId",
                                "ID of the ego-agent",
                                0]]
    return scenario

  def default_drone_model(self):
    param_server = ParameterServer()
    dynamic_model = SingleTrackModel(param_server)
    behavior_model = BehaviorConstantVelocity(param_server)
    execution_model = ExecutionModelInterpolate(param_server)
    drone_2d_shape = Polygon2d([0,0,0], np.array([
        [0.3,0.1],[0.3,-0.1],[0.1,-0.3],[-0.1,-0.3],[-0.3,-0.1],[-0.3,0.1],[-0.1,0.3],[0.1,0.3],[0.3,0.1]
    ]))
    init_state = np.array([0, 0, 0, 0, 0])

    drone_default = Agent(init_state,
                          behavior_model,
                          dynamic_model,
                          execution_model,
                          drone_2d_shape,
                          param_server)

    return drone_default


# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import unittest
import filecmp
import os
import matplotlib.pyplot as plt
from bark.core.geometry.standard_shapes import *
from bark.runtime.commons.parameters import ParameterServer
from bark.core.world.opendrive import XodrLaneType
from bark.core.world.map import MapInterface
from bark.runtime.commons.xodr_parser import XodrParser


def helper_plot(xodr_parser):
  for _, road in xodr_parser.map.GetRoads().items():
    for lane_section in road.lane_sections:
      for _, lane in lane_section.GetLanes().items():
        if lane.lane_type == XodrLaneType.driving:
          color = "grey"
        elif lane.lane_type == XodrLaneType.sidewalk:
          color = "green"
        elif lane.lane_type == XodrLaneType.border:
          color = "red"
        elif lane.lane_type == XodrLaneType.none:
          color = "blue"
        else:
          continue
        line_np = lane.line.ToArray()

        # print(lane.road_mark)
        plt.text(line_np[-1, 0], line_np[-1, 1],
                 'outer_{i}_{j}'.format(i=lane.lane_id, j=lane.lane_position))

        plt.plot(
            line_np[:, 0],
            line_np[:, 1],
            color=color,
            alpha=1.0)

  plt.axis("equal")
  plt.show()


class ImporterTests(unittest.TestCase):
  def test_map_highway(self):
    xodr_parser = XodrParser(os.path.join(os.path.dirname(__file__),"data/city_highway_straight.xodr"))

    map_interface = MapInterface()
    map_interface.SetOpenDriveMap(xodr_parser.map)
    # helper_plot(xodr_parser)


  def test_map_DR_DEU_Merging_MT_v01_shifted(self):
    xodr_parser = XodrParser(os.path.join(os.path.dirname(__file__),"data/DR_DEU_Merging_MT_v01_shifted.xodr"))

    map_interface = MapInterface()
    map_interface.SetOpenDriveMap(xodr_parser.map)
    # helper_plot(xodr_parser)


if __name__ == '__main__':
  unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import numpy as np
import time
import os
from bark.runtime.commons.parameters import ParameterServer
from bark.runtime.viewer.matplotlib_viewer import MPViewer
from bark.runtime.viewer.video_renderer import VideoRenderer
from bark.runtime.scenario.scenario_generation.config_with_ease import \
  LaneCorridorConfig, ConfigWithEase
from bark.runtime.runtime import Runtime
from bark.runtime.viewer.panda3d_easy import Panda3dViewer

from bark.core.world.opendrive import *
from bark.core.world.goal_definition import *
from bark.core.models.behavior import *

# parameters
param_server = ParameterServer()

# scenario
class CustomLaneCorridorConfig(LaneCorridorConfig):
  def __init__(self,
               params=None,
               **kwargs):
    super(CustomLaneCorridorConfig, self).__init__(params, **kwargs)
  
  def goal(self, world):
    road_corr = world.map.GetRoadCorridor(
      self._road_ids, XodrDrivingDirection.forward)
    lane_corr = self._road_corridor.lane_corridors[0]
    return GoalDefinitionPolygon(lane_corr.polygon)

param_server["BehaviorIDMClassic"]["BrakeForLaneEnd"] = True
param_server["BehaviorLaneChangeRuleBased"]["MinRemainingLaneCorridorDistance"] = 50.
param_server["BehaviorLaneChangeRuleBased"]["MinVehicleRearDistance"] = 2.
param_server["BehaviorLaneChangeRuleBased"]["MinVehicleFrontDistance"] = 2.
param_server["BehaviorLaneChangeRuleBased"]["TimeKeepingGap"] = 0.
param_server["BehaviorIDMClassic"]["DesiredVelocity"] = 10.

    
# configure both lanes of the highway. the right lane has one controlled agent
left_lane = CustomLaneCorridorConfig(params=param_server,
                                     lane_corridor_id=0,
                                     road_ids=[0, 1],
                                     behavior_model=BehaviorMobilRuleBased(param_server),
                                     s_min=5.,
                                     s_max=50.)
right_lane = CustomLaneCorridorConfig(params=param_server,
                                      lane_corridor_id=1,
                                      road_ids=[0, 1],
                                      controlled_ids=True,
                                      behavior_model=BehaviorMobilRuleBased(param_server),
                                      s_min=5.,
                                      s_max=20.)

scenarios = \
  ConfigWithEase(num_scenarios=3,
                 map_file_name= os.path.join(os.path.dirname(__file__), "../runtime/tests/data/DR_DEU_Merging_MT_v01_shifted.xodr"),
                 random_seed=0,
                 params=param_server,
                 lane_corridor_configs=[left_lane, right_lane])

# viewer
viewer = MPViewer(params=param_server,
                  x_range=[-35, 35],
                  y_range=[-35, 35],
                  follow_agent_id=True)
# viewer = Panda3dViewer(params=param_server,
#                        x_range=[-40, 40],
#                        y_range=[-40, 40],
#                        follow_agent_id=True,
#                        light_pose=[1000, 1000, 100000],
#                        camera_pose=[1000, 980, 100])

sim_step_time = param_server["simulation"]["step_time",
                                           "Step-time used in simulation",
                                           0.2]
sim_real_time_factor = param_server["simulation"]["real_time_factor",
                                                  "execution in real-time or faster",
                                                  1.]

# viewer = VideoRenderer(renderer=viewer,
#                        world_step_time=sim_step_time,
#                        fig_path="/Users/hart/2019/bark/video")

env = Runtime(step_time=0.2,
              viewer=viewer,
              scenario_generator=scenarios,
              render=True)

# run 3 scenarios
for _ in range(0, 3):
  env.reset()
  # step each scenario 20 times
  for step in range(0, 20):
    env.step()
    time.sleep(sim_step_time/sim_real_time_factor)

# viewer.export_video(filename="/Users/hart/2019/bark/video/video", remove_image_dir=False)



# Copyright (c) 2020 fortiss GmbH
#
# This software is released under the MIT License.
# https://opensource.org/licenses/MIT


class AgentTrackInfo:
    def __init__(self, filename, track_id, start_offset, end_offset, first_ts_on_map=None):
        self._filename = filename
        self._track_id = track_id  # TODO: rename to agent_id
        self._start_offset = start_offset  # TODO: rename to start_time
        self._end_offset = end_offset  # TODO: rename to end_time

    def __str__(self):
        return 'id={} start_offset={} end_offset={}'.format(self._track_id, self._start_offset, self._end_offset)

    def GetFileName(self):
        # yields track filename
        return self._filename

    def GetTrackId(self):
        return self._track_id

    def GetStartOffset(self):
        return self._start_offset

    def GetEndOffset(self):
        return self._end_offset



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import unittest
import pickle
import numpy as np

from bark.runtime.commons.parameters import ParameterServer
from bark.core.commons import CppParamServerTestObject
from bark.core.models.behavior import BehaviorIDMClassic

def pickle_unpickle(object):
    return pickle.loads(
      pickle.dumps(object))



class ParamServerTests(unittest.TestCase):
  def test_pickle(self):
    params = ParameterServer()
    params_child = params["test_child"]
    del params
    value_float = params_child["Child1"]["Child2"]["ValueFloat", "Desc", 2.0]
    value_bool_false = params_child["Child1"]["Child2"]["ValueBoolFalse", "Desc", False]
    value_bool_true = params_child["Child3"]["Child2"]["ValueBoolTrue", "Desc", True]
    value_int = params_child["Child1"]["Child4"]["ValueInt", "Desc", 2041]
    value_list_list_float = params_child["Child1"]["Child4"]["ValueListListFloat", "Desc",
         [[1.0, 2.0, float(5.4545234)], [float(1.1266135),2.0], [float(123.234234)]]]
    value_list_float = params_child["Child1"]["Child5"]["ValueListFloat", "Desc",
         [1.0, 2.0, float(5.4545234), float(1.1266135),2.0, float(123.234234)]]

    behavior = BehaviorIDMClassic(params_child)
    cpp_object = CppParamServerTestObject(params_child)
    cpp_unpickled = pickle_unpickle(cpp_object)

    self.assertEqual(cpp_object.GetRealValue(), 2.0)
    self.assertEqual(cpp_object.GetBoolValueFalse(), False)
    self.assertEqual(cpp_object.GetBoolValueTrue(), True)
    self.assertEqual(cpp_object.GetIntValue(), 2041)
    list1 = cpp_object.GetListListFloatValue()
    list2 =[[1.0,2.0, float(5.4545234)], [float(1.1266135),2.0], [float(123.234234)]]

    list3 = cpp_object.GetListFloatValue()
    list4 =[1.0, 2.0, float(5.4545234), float(1.1266135),2.0, float(123.234234)]

    self.assertEqual(len(list1), len(list2))
    for idx, _ in enumerate(list1):
      self.assertEqual(len(list1[idx]), len(list2[idx]))
      for idx2, _ in enumerate(list1[idx]):
        self.assertAlmostEqual(list1[idx][idx2], list2[idx][idx2], places=5)

    self.assertEqual(len(list3), len(list4))
    for idx, _ in enumerate(list3):
        self.assertAlmostEqual(list3[idx], list4[idx], places=5)

  def test_key_not_found(self):
    params = ParameterServer(log_if_default=True)
    params_child = params["test_child"]
    del params
    params_child["Child1"]["Child2"]["ValueFloat"] = 2.0
    params_child["Child1"]["Child10"]["ValueBoolFalse"] = False

    cpp_object = CppParamServerTestObject(params_child)



if __name__ == '__main__':
  unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import unittest
import time
import math
import filecmp
import matplotlib.pyplot as plt
from bark.core.world.agent import Agent
from bark.core.world import World
from bark.core.geometry import Point2d, Polygon2d
from bark.core.geometry import Point2d
from bark.runtime.commons.parameters import ParameterServer
from bark.core.world.opendrive import OpenDriveMap, MakeXodrMapOneRoadTwoLanes, \
  XodrLaneType
from bark.core.world.map import MapInterface
from bark.runtime.commons import XodrParser
from bark.runtime.viewer import MPViewer
import numpy as np
import os

class EnvironmentTests(unittest.TestCase):
  def test_two_roads_one_lane(self):
    params = ParameterServer()
    world = World(params)

    xodr_map = MakeXodrMapOneRoadTwoLanes()

    map_interface = MapInterface()
    map_interface.SetOpenDriveMap(xodr_map)
    world.SetMap(map_interface)

    start_point = Point2d(0, -11)
    lanes_near_start = map_interface.find_nearest_lanes(start_point, 1)
    assert(len(lanes_near_start) == 1)

    goal_point = Point2d(-191.789, -50.1725)
    lanes_near_goal = map_interface.find_nearest_lanes(goal_point, 1)
    assert(len(lanes_near_goal) == 1)

    viewer = MPViewer(params=params, use_world_bounds=True)
    viewer.drawWorld(world)

    time.sleep(2)  # if this is not here, the second unit test is not executed (maybe parsing takes too long?)

  def test_between_lanes(self):
    xodr_parser = XodrParser(os.path.join(os.path.dirname(__file__),"../../runtime/tests/data/city_highway_straight.xodr"))
    np.set_printoptions(precision=8)
    params = ParameterServer()

    world = World(params)
    map_interface = MapInterface()
    map_interface.SetOpenDriveMap(xodr_parser.map)
    world.SetMap(map_interface)

    # Simple test
    point_close = Point2d(5114.68262, 5086.44971)
    lane_sw = map_interface.FindLane(point_close)
    self.assertIsNotNone(lane_sw, "This point is still in the left lane! XodrLane boundary is 5114.683")

    switched_lane = False
    lng_coord = 5086.44971
    i = 5114.5
    lane_sw = map_interface.FindLane(Point2d(i, lng_coord))
    assert lane_sw != None
    prev = lane_sw.lane_id
    prev_i = i
    while (i < 5117.5):
      lane_sw = map_interface.FindLane(Point2d(i, lng_coord))
      self.assertIsNotNone(lane_sw, "Should always be on at least one lane! Currently at ({}, {})".format(i, lng_coord))
      if prev != lane_sw.lane_id:
        # print(prev)
        # print(prev_i)
        # print(lane_sw.lane_id)
        # print(i)
        self.assertFalse(switched_lane, "XodrLane switch should only happens once!")
        switched_lane = True
      prev_i = i
      prev = lane_sw.lane_id
      i = i + 0.01
    self.assertTrue(switched_lane, "Eventually should have switched lanes!")

if __name__ == '__main__':
  unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

from bark.runtime.scenario import Scenario
from bark.runtime.scenario.scenario_generation import ScenarioGeneration
from bark.runtime.commons import ModelJsonConversion
from bark.core.world.agent import *
from bark.core.models.behavior import *
from bark.core.world import *
from bark.core.world.goal_definition import GoalDefinition, GoalDefinitionPolygon, GoalDefinitionStateLimits
from bark.core.world.map import *
from bark.core.models.dynamic import *
from bark.core.models.execution import *
from bark.core.geometry import *
from bark.core.geometry.standard_shapes import *
from bark.runtime.commons import ParameterServer
from bark.runtime.commons import XodrParser

import numpy as np
import math


class UniformVehicleDistribution(ScenarioGeneration):
  def __init__(self, num_scenarios, params=None, random_seed=None):
    super(UniformVehicleDistribution, self).__init__(params, num_scenarios)
    self.initialize_params(params)

  def initialize_params(self, params):
    params_temp = \
      self._params["Scenario"]["Generation"]["UniformVehicleDistribution"]
    self._random_seed = 1000 # since ScenarioGeneration UniformVehicleDistribution
      # will soon be deprecated we introduce this hack
    self._map_file_name = params_temp["MapFilename",
      "Path to the open drive map", 
      "bark/runtime/tests/data/city_highway_straight.xodr",    ]
    self._ego_goal_end = params_temp["EgoGoalEnd",
      "The center of the ego agent's goal region polygon",
      [5117, 5200] ]
    self._ego_goal_start = params_temp["EgoGoalStart",
      "The coordinates of the start of the ego goal,\
           if empty only ego goal end is used as center of polygon ",
      [] ]
    self._ego_goal_state_limits = params_temp["EgoGoalStateLimits",
      "x,y and theta limits around center line of lane between start and end applied to both lateral sides \
       (only valid if start and end goal of ego are given)",
       [0.1, 0, 0.08]]
    self._ego_route = params_temp["EgoRoute",
      "A list of two points defining start and end point of initial ego driving corridor. \
           If empty, then one of the other agents is selected as ego agents.",
      [[5117.5, 5100], [5117.5, 5200]]]
    self._others_source = params_temp["OthersSource",
      "A list of points around which other vehicles spawn. \
        Points should be on different lanes. XodrLanes must be near these points \
      (<0.5m) Provide a list of lists with x,y-coordinates",
     [[5114.626, 5061.8305]]]
    self._others_sink = params_temp["OthersSink",
      "A list of points defining end of other vehicles routes.\
        Points should be on different lanes and match the order of the\
        source points. XodrLanes must be near these points (<0.5m) \
        Provide a list of lists with x,y-coordinates",
        [[ 5114.626, 5193.1725]] ]  
    assert len(self._others_sink) == len(self._others_source)         
    self._vehicle_distance_range = params_temp["VehicleDistanceRange",
      "Distance range between vehicles in meter given as tuple from which" + \
      "distances are sampled uniformly",
      (10, 20)]
    self._other_velocity_range = params_temp["OtherVehicleVelocityRange",
      "Lower and upper bound of velocity in km/h given as tuple from which" + \
      " velocities are sampled uniformly",
      (20,30)]
    self._ego_velocity_range = params_temp["EgoVehicleVelocityRange",
      "Lower and upper bound of velocity in km/h given as tuple from which" + \
      " velocities are sampled uniformly",
      (20,30)]
    json_converter = ModelJsonConversion()
    self._agent_params = params_temp["VehicleModel",
      "How to model the other agents",
      json_converter.agent_to_json(self.default_agent_model())]
    if not isinstance(self._agent_params, dict):
        self._agent_params = self._agent_params.ConvertToDict()
    np.random.seed(self._random_seed)


  def create_scenarios(self, params, num_scenarios):
    """ 
        see baseclass
    """
    scenario_list = []
    for scenario_idx in range(0, num_scenarios):
      scenario = self.create_single_scenario()     
      scenario_list.append(scenario)
    return scenario_list

  def create_single_scenario(self):
    scenario = Scenario(map_file_name=self._map_file_name,
                        json_params=self._params.ConvertToDict())
    world = scenario.GetWorldState()
    agent_list = []
    # OTHER AGENTS
    for idx, source in enumerate(self._others_source):
      connecting_center_line, s_start, s_end = \
        self.center_line_between_source_and_sink(world.map,
                                                 source,
                                                 self._others_sink[idx])
      goal_polygon = Polygon2d([0, 0, 0],
                               [Point2d(-1.5,0),
                                Point2d(-1.5,8),
                                Point2d(1.5,8),
                                Point2d(1.5,0)])
      goal_polygon = goal_polygon.Translate(Point2d(self._others_sink[idx][0],
                                                    self._others_sink[idx][1]))
      goal_definition = GoalDefinitionPolygon(goal_polygon)
      agent_list.extend(
        self.place_agents_along_linestring(world,
                                           connecting_center_line,
                                           s_start,
                                           s_end,
                                           self._agent_params,
                                           goal_definition))

    description=self._params.ConvertToDict()
    description["ScenarioGenerator"] = "UniformVehicleDistribution"
    

    # EGO AGENT
    ego_agent=None
    if len(self._ego_route) == 0:
        # take agent in the middle of list 
        num_agents = len(agent_list)
        ego_agent = agent_list[math.floor(num_agents/4)] 
    else:
        connecting_center_line, s_start, s_end  = \
        self.center_line_between_source_and_sink(world.map,
                                                 self._ego_route[0],
                                                 self._ego_route[1])

        sego = self.sample_srange_uniform([s_start, s_end])
        xy_point =  GetPointAtS(connecting_center_line, sego)
        angle = GetTangentAngleAtS(connecting_center_line, sego)
        velocity = self.sample_velocity_uniform(self._ego_velocity_range)
        agent_state = np.array([0, xy_point.x(), xy_point.y(), angle, velocity ])

        agent_params = self._agent_params.copy()
        agent_params["state"] = agent_state
        # goal for driving corridor generation
        goal_polygon = Polygon2d([0, 0, 0],
                               [Point2d(-1.5,0),
                                Point2d(-1.5,8),
                                Point2d(1.5,8),
                                Point2d(1.5,0)])
        goal_polygon = goal_polygon.Translate(Point2d(self._ego_route[1][0],
                                                    self._ego_route[1][1]))
        goal_definition = GoalDefinitionPolygon(goal_polygon)
        agent_params["goal_definition"] = goal_definition
        agent_params["map_interface"] = world.map

        converter = ModelJsonConversion()
        ego_agent = converter.agent_from_json(agent_params, self._params["Agent"])
        # TODO(@bernhard): ensure that ego agent not collides with others
    
    agent_list.append(ego_agent)


    
    # EGO Agent Goal Definition
    if  len(self._ego_goal_start) == 0:
        if len(self._ego_route) == 0:
          # ego agent is one of the random agents, so the goal definition is
          # already set
          pass
        else:
          goal_polygon = Polygon2d([0, 0, 0],
                                   [Point2d(-1.5,0),
                                    Point2d(-1.5,8),
                                    Point2d(1.5,8),
                                    Point2d(1.5,0)])
          goal_polygon = goal_polygon.Translate(Point2d(self._ego_goal_end[0],
                                                      self._ego_goal_end[1]))
          ego_agent.goal_definition = GoalDefinitionPolygon(goal_polygon)
    else:
        connecting_center_line, s_start, s_end = \
        self.center_line_between_source_and_sink(world.map,
                                                 self._ego_goal_start,
                                                 self._ego_goal_end)

        goal_center_line = GetLineFromSInterval(connecting_center_line, s_start, s_end)

        # build polygon representing state limits
        lims = self._ego_goal_state_limits
        goal_limits_left = goal_center_line.Translate(Point2d(-lims[0], -lims[1]))
        goal_limits_right = goal_center_line.Translate(Point2d(lims[0], lims[1]))
        goal_limits_right.Reverse()
        goal_limits_left.AppendLinestring(goal_limits_right)
        polygon = Polygon2d([0,0,0], goal_limits_left)

        ego_agent.goal_definition = GoalDefinitionStateLimits(polygon, (1.57-0.08, 1.57+0.08))

    # only one agent is ego in the middle of all other agents
    scenario._agent_list = agent_list
    scenario._eval_agent_ids = [ego_agent.id]
    return scenario

  def place_agents_along_linestring(self,
                                    world,
                                    linestring,
                                    s_start,
                                    s_end,
                                    agent_params,
                                    goal_definition):
    s = s_start
    if s_end < s_start:
      linestring.Reverse()
      s = s_end
      s_end = s_start
    agent_list = []
    while s < s_end:
      # set agent state on linestring with random velocity
      xy_point =  GetPointAtS(linestring, s)
      angle = GetTangentAngleAtS(linestring, s)
      
      velocity = self.sample_velocity_uniform(self._other_velocity_range)
      agent_state = np.array([0, xy_point.x(), xy_point.y(), angle, velocity ])

      agent_params = self._agent_params.copy()
      agent_params["state"] = agent_state
      agent_params["goal_definition"] = goal_definition
      agent_params["map_interface"] = world.map

      converter = ModelJsonConversion()
      bark_agent = converter.agent_from_json(agent_params, self._params["Agent"])
      agent_list.append(bark_agent)

      # move forward on linestring based on vehicle size and max/min distance
      s += bark_agent.shape.front_dist + bark_agent.shape.rear_dist + \
                  self.sample_distance_uniform(self._vehicle_distance_range)
    return agent_list


  def sample_velocity_uniform(self, velocity_range):
    return np.random.uniform(velocity_range[0], velocity_range[1])

  def sample_distance_uniform(self, distance_range):
    return np.random.uniform(distance_range[0], distance_range[1])

  def sample_srange_uniform(self, srange):
    return np.random.uniform(srange[0], srange[1])

  def center_line_between_source_and_sink(self, map_interface, source, sink):
    # generate road corridor between source and sink
    goal_polygon = Polygon2d([0, 0, 0],
                                  [Point2d(-1,0),
                                  Point2d(-1,1),
                                  Point2d(1,1),
                                  Point2d(1,0)])
    goal_polygon = goal_polygon.Translate(Point2d(sink[0],
                                                      sink[1]))
    road_corridor = map_interface.GenerateRoadCorridor(
        Point2d(source[0], source[1]),
        goal_polygon)

    # find lane corridor between source and sink
    lane_corridor_sink = road_corridor.GetCurrentLaneCorridor(Point2d(sink[0],
                                                      sink[1]))
    lane_corridor_source = road_corridor.GetCurrentLaneCorridor(Point2d(source[0],
                                                      source[1]))

    if (not lane_corridor_sink or not lane_corridor_source) or  lane_corridor_sink != lane_corridor_source:
      print("source and sink lane corridors not equal!.")
      return

    center_line = lane_corridor_sink.center_line

    _, s_start, _ = GetNearestPointAndS(center_line,
                                            Point2d(source[0],source[1]))
    _, s_end, _ = GetNearestPointAndS(center_line,
                                          Point2d(sink[0],sink[1]))
    return center_line, \
           s_start, \
           s_end


  def default_agent_model(self):
    param_server = ParameterServer()
    behavior_model = BehaviorConstantVelocity(param_server)
    execution_model = ExecutionModelInterpolate(param_server)
    dynamic_model = SingleTrackModel(param_server)
    map_interface = MapInterface()

    agent_2d_shape = CarLimousine()
    init_state = np.array([0, 0, 0, 0, 0])

    agent_default = Agent(init_state,
                          behavior_model,
                          dynamic_model,
                          execution_model,
                          agent_2d_shape,
                          param_server)

    return agent_default



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

from bark.runtime.scenario import Scenario
from bark.runtime.scenario.scenario_generation  import ScenarioGeneration
from bark.runtime.scenario.scenario_generation.config_readers import *
from bark.runtime.commons.parameters import ParameterServer

from bark.core.geometry import *
from bark.core.world.agent import Agent

import numpy as np
import math
import copy
import aabbtree
from collections import defaultdict 


class ConfigurableScenarioGeneration(ScenarioGeneration):
  def __init__(self, num_scenarios, params=None, random_seed=1000):
    super(ConfigurableScenarioGeneration, self).__init__(params, num_scenarios, random_seed)

  def initialize_params(self, params):
    print (params["Scenario"])
    params_temp = \
      self._params["Scenario"]["Generation"]["ConfigurableScenarioGeneration"]
    self._map_file_name = params_temp["MapFilename",
      "Path to the open drive map", 
      "bark/runtime/tests/data/city_highway_straight.xodr",    ]
    self._sinks_sources = params_temp["SinksSources", "Random seed used for sampling", [{
      "SourceSink": [[5111.626, 5006.8305],  [5110.789, 5193.1725] ],
      "Description": "left_lane",
      "ConfigAgentStatesGeometries": {"Type": "UniformVehicleDistribution", "LanePositions": [0]},
      "ConfigBehaviorModels": {"Type": "FixedBehaviorType", "ModelType" : "BehaviorIDMClassic", "ModelParams" :  {"BehaviorIDMClassic::MaxVelocity" : 60.0}},
      "ConfigExecutionModels": {"Type": "FixedExecutionType"},
      "ConfigDynamicModels": {"Type": "FixedDynamicType"},
      "ConfigGoalDefinitions": {"Type": "FixedGoalTypes"},
      "ConfigControlledAgents": {"Type": "NoneControlled"},
      "AgentParams" : {}
    },
    {
      "SourceSink": [[5111.626, 5006.8305],  [5110.789, 5193.1725] ],
      "Description": "right_lane",
      "ConfigAgentStatesGeometries": {"Type": "UniformVehicleDistribution", "LanePositions": [1]},
      "ConfigBehaviorModels": {"Type": "FixedBehaviorType", "ModelType" : "BehaviorIDMClassic", "ModelParams" :  {"BehaviorIDMClassic::MaxVelocity" : 30.0}},
      "ConfigExecutionModels": {"Type": "FixedExecutionType"},
      "ConfigDynamicModels": {"Type": "FixedDynamicType"},
      "ConfigGoalDefinitions": {"Type": "FixedGoalTypes"},
      "ConfigControlledAgents": {"Type": "RandomSingleAgent"},
      "AgentParams" : {}
    }
    ]
    ]

    print (self._map_file_name)
    self._conflict_resolutions = params_temp["ConflictResolution", "How are conflicts for overlapping \
              sources and sinks resolved", {"left_lane/right_lane" : (0.2, 0.8)}]
    self._random_state = np.random.RandomState(self._random_seed)

    # all parameter servers used by all bark class instances must be persisted
    # otherwise parameter server serialization fails
    self._sink_source_parameter_servers = defaultdict(list)

    self._sink_source_default_params = None

    self._registered_types = {}

  def update_defaults_params(self):
    self._params["Scenario"]["Generation"]["ConfigurableScenarioGeneration"]["SinksSources"] = \
        self._sink_source_default_params

  def add_config_reader_parameter_servers(self, description, config_type, config_reader):
    self._sink_source_parameter_servers[config_type].append(
      {"Description": description,
       "ParameterServers" : config_reader.get_param_servers()}
    )

  def create_scenarios(self, params, num_scenarios):
    """ 
        see baseclass
    """
    scenario_list = []
    for scenario_idx in range(0, num_scenarios):
      scenario = self.create_single_scenario()     
      scenario_list.append(scenario)
    self.update_defaults_params()
    return scenario_list

  def create_single_scenario(self):
    scenario = Scenario(map_file_name=self._map_file_name,
                        json_params=self._params.ConvertToDict())
    world = scenario.GetWorldState()
    collected_sources_sinks_agent_states_geometries = []
    collected_sources_sinks_default_param_configs = []

    # Loop through each source sink config and first only create state
    # and geometry information
    road_corridors = []
    kwargs_agent_states_geometry = []
    sink_source_default_params = []
    for idx, sink_source_config in enumerate(self._sinks_sources):
      road_corridor = self.get_road_corridor_from_source_sink(sink_source_config, world.map)
      road_corridors.append(road_corridor)
      
      #1) create agent states and geometries for this source
      args = [road_corridor]
      agent_states, agent_geometries, kwargs_dict, default_params_state_geometry = \
        self.eval_configuration( sink_source_config, "ConfigAgentStatesGeometries",
                              args, {})
      kwargs_agent_states_geometry.append(kwargs_dict)

      # collect default parameters of this config
      sink_source_default_params.append(sink_source_config)
      sink_source_default_params[idx]["ConfigAgentStatesGeometries"] = default_params_state_geometry.ConvertToDict()
      collected_sources_sinks_agent_states_geometries.append((agent_states, agent_geometries))

    #2 remove overlapping agent states from different sources and sinks
    collected_sources_sinks_agent_states_geometries = \
            self.resolve_overlaps_in_sources_sinks_agents(collected_sources_sinks_agent_states_geometries)

    agent_list = []
    controlled_agent_ids_all = []
    for idx, agent_states_geometries in enumerate(
                    collected_sources_sinks_agent_states_geometries):
      sink_source_config = self._sinks_sources[idx]
      agent_states = agent_states_geometries[0]
      agent_geometries = agent_states_geometries[1]
      road_corridor = road_corridors[idx]

      if(len(agent_states)== 0):
        continue

      #3) create behavior, execution and dynamic models
      args_list = [road_corridor, agent_states ]
      kwargs_dict = {**kwargs_agent_states_geometry[idx]}
      config_return, kwargs_dict_tmp, default_params_behavior = \
        self.eval_configuration(
                              sink_source_config, "ConfigBehaviorModels", 
                              args_list, kwargs_dict)
      behavior_models = config_return
      sink_source_default_params[idx]["ConfigBehaviorModels"] = default_params_behavior.ConvertToDict()
      
      kwargs_dict = {**kwargs_dict, **kwargs_dict_tmp}
      config_return, kwargs_dict_tmp, default_params_execution = \
        self.eval_configuration(
                              sink_source_config, "ConfigExecutionModels",
                              args_list, kwargs_dict)
      execution_models = config_return
      sink_source_default_params[idx]["ConfigExecutionModels"] = default_params_execution.ConvertToDict()
      kwargs_dict = {**kwargs_dict, **kwargs_dict_tmp}


      config_return, kwargs_dict_tmp, default_params_dynamic = \
        self.eval_configuration(
                              sink_source_config, "ConfigDynamicModels",
                              args_list, kwargs_dict)
      dynamic_models = config_return
      sink_source_default_params[idx]["ConfigDynamicModels"] = default_params_dynamic.ConvertToDict()
      kwargs_dict = {**kwargs_dict, **kwargs_dict_tmp}

      #4 create goal definitions and controlled agents
      config_return, kwargs_dict_tmp, default_params_controlled_agents = \
        self.eval_configuration(
                              sink_source_config, "ConfigControlledAgents",
                              args_list, kwargs_dict)
      controlled_agent_ids = config_return
      controlled_agent_ids_all.extend(controlled_agent_ids)
      sink_source_default_params[idx]["ConfigControlledAgents"] = default_params_controlled_agents.ConvertToDict()
      kwargs_dict = {**kwargs_dict, **kwargs_dict_tmp}

      args_list = [*args_list, controlled_agent_ids]
      config_return, kwargs_dict_tmp, default_params_goals = \
        self.eval_configuration(
                              sink_source_config, "ConfigGoalDefinitions", 
                              args_list, kwargs_dict)
      goal_definitions = config_return
      sink_source_default_params[idx]["ConfigGoalDefinitions"] = default_params_goals.ConvertToDict()

      #5 Build all agents for this source config
      agent_params = ParameterServer(json = sink_source_config["AgentParams"])
      sink_source_agents = self.create_source_config_agents(agent_states,
                      agent_geometries, behavior_models, execution_models,
                      dynamic_models, goal_definitions, controlled_agent_ids,
                      world, agent_params)
      sink_source_default_params[idx]["AgentParams"] = agent_params.ConvertToDict()

      agent_list.extend(sink_source_agents)
      collected_sources_sinks_default_param_configs.append(sink_source_config)

    self._sink_source_default_params = sink_source_default_params
    scenario._agent_list = self.update_agent_ids(agent_list)
    scenario._eval_agent_ids = [idx for idx, value in enumerate(controlled_agent_ids_all) if value==True]
    
    return scenario

  def update_agent_ids(self, agent_list):
    for idx, agent in enumerate(agent_list):
      agent.SetAgentId(idx)
    return agent_list

  def resolve_overlaps_in_sources_sinks_agents(self, 
                  collected_sources_sinks_agent_states_geometries):

      collisions = ConfigurableScenarioGeneration.find_overlaps_in_sources_sinks_agents(
        collected_sources_sinks_agent_states_geometries
      )
      return self.delete_overlaps_in_sources_sinks_agents(
        collected_sources_sinks_agent_states_geometries,
        collisions
      )

  @staticmethod
  def find_overlaps_in_sources_sinks_agents( 
                  collected_sources_sinks_agent_states_geometries):
    # create aabbtree for fast distance lookup between agents
    tree = aabbtree.AABBTree()
    for source_sink_idx, states_geometries in enumerate(
            collected_sources_sinks_agent_states_geometries):
      agent_states = states_geometries[0]
      agent_geometries = states_geometries[1]
      for agent_idx, agent_state in enumerate(agent_states):
        agent_translated_polygon = agent_geometries[agent_idx].Translate(
                                                      Point2d(agent_state[1],
                                                            agent_state[2]))
        tmp = agent_translated_polygon.bounding_box
        bb = aabbtree.AABB([(tmp[0].x(), tmp[1].x()), (tmp[0].y(), tmp[1].y())])
        tree.add(bb, (source_sink_idx, agent_idx))

    # check for all agents in a source sink config collision with
    # agents in other sources sinks and track the collisions
    collisions = defaultdict(list)
    for source_sink_idx, states_geometries in enumerate(
            collected_sources_sinks_agent_states_geometries):
        agent_states = states_geometries[0]
        agent_geometries = states_geometries[1]
        for agent_idx, agent_state in enumerate(agent_states):
          agent_translated_polygon = agent_geometries[agent_idx].Translate(
                                                        Point2d(agent_state[1],
                                                              agent_state[2]))
          tmp = agent_translated_polygon.bounding_box
          bb = aabbtree.AABB([(tmp[0].x(), tmp[1].x()), ( tmp[0].y() , tmp[1].y())])
          overlaps = tree.overlap_values(bb)
          for overlap in overlaps:
            if source_sink_idx == overlap[0]:
              if agent_idx == overlap[1]:
                continue
              else:
                raise ValueError("Something went wrong. \
                    We have colliding agent within one source sink configuration")
          
            key1 = "{}-{}".format(source_sink_idx, overlap[0])
            key2 = "{}-{}".format(overlap[0], source_sink_idx, )
            key = None
            if key1 in collisions:
              key = key1
            elif key2 in collisions:
              key = key2
            else:
              key = key1

            pairwise_collisions = collisions[key]
            found = False
            for collision in pairwise_collisions:
              # exclude the case where this collision was already detected in a previous
              # run through the loop
              agent_desc1 = collision[0]
              if overlap[0] == agent_desc1[0] and overlap[1] == agent_desc1[1]:
                found = True
            if not found:
              agent_geometry_other = collected_sources_sinks_agent_states_geometries[overlap[0]][1][overlap[1]]
              agent_state_other = collected_sources_sinks_agent_states_geometries[overlap[0]][0][overlap[1]]
              agent_translated_polygon_other = agent_geometry_other.Translate(
                                              Point2d(agent_state_other[1],
                                                    agent_state_other[2]))
              if Collide(agent_translated_polygon, agent_translated_polygon_other):
                pairwise_collisions.append(((source_sink_idx, agent_idx), overlap))
            
                collisions[key] = pairwise_collisions
    
    return collisions

  def delete_overlaps_in_sources_sinks_agents(self, 
                  collected_sources_sinks_agent_states_geometries, collisions):
    # first find all agents to delete, then delete in go
    must_delete = defaultdict(list)
    np.random.seed(self._random_seed)
    for key, pairwise_collisions in collisions.items():
      # get the conflict resolution scheme defined using descriptions in src sink configs  "left_turn/straight": 0.2:0.8
      source_sink_idx_1 = key.split("-")[0]
      source_sink_idx_2 = key.split("-")[1]

      source_sink_desc_1 = self._sinks_sources[int(source_sink_idx_1)]["Description"]
      source_sink_desc_2 = self._sinks_sources[int(source_sink_idx_2)]["Description"]
         
      num_conflict_res1, num_conflict_res2, probablistic_conflict_resolution = \
         self.parse_conflict_resolution(source_sink_desc_1, source_sink_desc_2)

      if not probablistic_conflict_resolution:
        raise NotImplemented("Not implemented yet.")
      
      for pairwise_collision in pairwise_collisions:
        source_sink_idx_1 = pairwise_collision[0][0]
        source_sink_idx_2 = pairwise_collision[1][0]
        agent_idx_1 = pairwise_collision[0][1]
        agent_idx_2 = pairwise_collision[1][1]

        if probablistic_conflict_resolution:
          sample = np.random.rand()
          if 0 <= sample and sample <= num_conflict_res1:
            if not agent_idx_1 in must_delete[source_sink_idx_1]:
              must_delete[source_sink_idx_1].append(agent_idx_1)
          elif num_conflict_res1 < sample <= (num_conflict_res1+num_conflict_res2):
            if not agent_idx_2 in must_delete[source_sink_idx_2]:
              must_delete[source_sink_idx_2].append(agent_idx_2)
          else:
            if not agent_idx_1 in must_delete[source_sink_idx_1]:
              must_delete[source_sink_idx_1].append(agent_idx_1)
            if not agent_idx_2 in must_delete[source_sink_idx_2]:
              must_delete[source_sink_idx_2].append(agent_idx_2)

    for source_sink_idx, agent_deletions in enumerate(must_delete):
      agent_list = collected_sources_sinks_agent_states_geometries[source_sink_idx]
      if not isinstance(agent_deletions, list):
        agent_deletions = [agent_deletions]
      for index in list(sorted(agent_deletions, reverse=True)):
        del agent_list[0][index]
        del agent_list[1][index]

    return collected_sources_sinks_agent_states_geometries

  def parse_conflict_resolution(self, description1, description_2):
    # find for two src sink configs the conflict resolution and parse it
    key = self.conflict_resolution_key_from_src_conf_desc(description1, description_2)
    key_reverse = self.conflict_resolution_key_from_src_conf_desc(description_2, description1)
    reversed = False
    if not key in self._conflict_resolutions:
      if key_reverse in self._conflict_resolutions:
        key = key_reverse
        reversed = True
      else:
        raise ValueError("Conflict resolution scheme for {} and {} not specified.".format(desc1, desc2))
    conflict_res_1 = self._conflict_resolutions[key][0]
    conflict_res_2 = self._conflict_resolutions[key][1]
    if reversed:
      tmp = conflict_res_1
      conflict_res_1 = conflict_res_2
      conflict_res_2 = tmp

    def int_or_float(s):
      try:
        int(s)
        return True, int(s)
      except:
        try:
          float(s)
          return False, float(s)
        except:
          raise ValueError("Could not parse conflict resolution {}".format(s))
    
    num_conflict_res1, is_int_1 = int_or_float(conflict_res_1)
    num_conflict_res2, is_int_2 = int_or_float(conflict_res_1)

    if is_int_1 and is_int_2:
      probablistic_conflict_resolution = False
    elif not is_int_1 and not is_int_2:
      probablistic_conflict_resolution = True
    else:
      raise ValueError("Conflict resolution specifications must be either both integers or both floats")

    return num_conflict_res1, num_conflict_res2, probablistic_conflict_resolution


  def find_src_conf_idx_from_desc(self, desc):
    idx = 0
    for src_conf in self._sinks_sources:
      if desc == src_conf["Description"]:
        return idx
      idx += 1
    raise ValueError("Description not found in source sink configs.")
  
  @staticmethod
  def conflict_resolution_key_from_src_conf_desc(desc1, desc2):
    return "{}/{}".format(desc1, desc2)


  def create_source_config_agents(self, agent_states, agent_geometries, 
                        behavior_models, execution_models, dynamic_models,
                        goal_definitions, controlled_agent_ids, world, agent_params):
    num_agents = len(agent_states)
    if any(len(lst) != num_agents for lst in [
      agent_geometries, behavior_models, execution_models, dynamic_models, goal_definitions, controlled_agent_ids]):
      raise ValueError("Config readers did not return equal sized of lists")
    agents = []
    for idx, agent_state in enumerate(agent_states):
      bark_agent = Agent( np.array(agent_state), 
                          behavior_models[idx], 
                          dynamic_models[idx],
                          execution_models[idx], 
                          agent_geometries[idx],
                          agent_params,
                          goal_definitions[idx],
                          world.map )

      agents.append(bark_agent)
    
    return agents

  def eval_configuration(self, sink_source_config, config_type, args, kwargs):
    eval_config = sink_source_config[config_type]
    eval_config_type = eval_config["Type"]
    param_config = ParameterServer(json = eval_config)
    config_reader = eval("{}(self._random_state)".format(eval_config_type))
    config_return  = config_reader.create_from_config(param_config, *args, **kwargs)
    self.add_config_reader_parameter_servers(sink_source_config["Description"], config_type, config_reader)
    return config_return

  @staticmethod
  def get_road_corridor_from_source_sink(source_sink_properties, map_interface):
    # generate road corridor between source and sink
    source_sink = source_sink_properties["SourceSink"]
    road_corridor = None
    if isinstance(source_sink, tuple) and \
           isinstance(source_sink[0], int):
           # road id given for start and end of road corridor 
           start_road_id = source_sink[0]
           end_road_id = source_sink[1]
           road_corridor = map_interface.GenerateRoadCorridor(start_road_id,
                                                end_road_id)
    elif isinstance(source_sink, list) and \
           isinstance(source_sink[0], list)  and \
           isinstance(source_sink[0][0], float):
           # point 2d given to find start and end road id road corridor
            goal_polygon = Polygon2d([0, 0, 0],
                                  [Point2d(-1,0),
                                  Point2d(-1,1),
                                  Point2d(1,1),
                                  Point2d(1,0)])
            start_point = Point2d(source_sink[0][0], source_sink[0][1])
            end_point = Point2d(source_sink[1][0], source_sink[1][1])
            goal_polygon = goal_polygon.Translate(end_point)
            road_corridor = map_interface.GenerateRoadCorridor(start_point,
                                                goal_polygon)
    else:
      raise ValueError("Unknown specification of start and end \
              of source sink config {}".format(source_sink))
    
    if not road_corridor:
      raise ValueError("No road corridor found betwen source {} \
              "" and sink {}".format(source_sink[0], source_sink[1]) )

    return road_corridor



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

import unittest
from bark.runtime.scenario.interaction_dataset_processing.interaction_dataset_reader import agent_from_trackfile
from bark.runtime.scenario.interaction_dataset_processing.agent_track_info import AgentTrackInfo
from bark.runtime.scenario.interaction_dataset_processing.scenario_track_info import ScenarioTrackInfo
from bark.runtime.commons.parameters import ParameterServer
import os

class InteractionDatasetReaderTest(unittest.TestCase):

    def test_scenario_track_info(self):
        map_filename = os.path.join(os.path.dirname(__file__),"../runtime/tests/data/DR_DEU_Merging_MT_v01_shifted.xodr")
        track_filename = os.path.join(os.path.dirname(__file__),"../runtime/tests/data/interaction_dataset_dummy_track.csv")

        agent1 = AgentTrackInfo(
            track_filename, track_id=1, start_offset=500, end_offset=1000)

        print(agent1)

        agent2 = AgentTrackInfo(
            track_filename, track_id=2, start_offset=500, end_offset=1000)

        print(agent2)

        scen_info = ScenarioTrackInfo(
            map_filename, track_filename, agent1, start_ts=500, end_ts=1000)
        scen_info.AddTrackInfoOtherAgent(agent2)

        print(scen_info)

        assert(scen_info.TimeSanityCheck() == True)

    def test_agent_from_trackfile(self):

        map_filename = os.path.join(os.path.dirname(__file__),"../tests/data/DR_DEU_Merging_MT_v01_shifted.xodr")
        track_filename = os.path.join(os.path.dirname(__file__),"../tests/data/interaction_dataset_dummy_track.csv")

        agent_track_info = AgentTrackInfo(
            track_filename, track_id=1, start_offset=500, end_offset=1000)

        params = ParameterServer()
        params_id = params["Scenario"]["Generation"]["InteractionDataset"]
        params_id["MapFilename", "",
                  os.path.join(os.path.dirname(__file__),"../tests/data/DR_DEU_Merging_MT_v01_shifted.xodr")]
        params_id["TrackFilename", "",
                  os.path.join(os.path.dirname(__file__),"../tests/data/interaction_dataset_dummy_track.csv")]
        params_id["BehaviorModel", "", {}]

        track_params = ParameterServer()
        track_params["execution_model"] = 'ExecutionModelInterpolate'
        track_params["dynamic_model"] = 'SingleTrackModel'
        track_params["map_interface"] = None  # world.map
        track_params["behavior_model"] = None

        scenario_info = ScenarioTrackInfo(
            map_filename, track_filename, agent_track_info)

        agent = agent_from_trackfile(
            track_params, params, scenario_info, agent_track_info.GetTrackId())


if __name__ == '__main__':
    unittest.main()



# Copyright (c) 2020 Julian Bernhard, Klemens Esterle, Patrick Hart and
# Tobias Kessler
#
# This work is licensed under the terms of the MIT license.
# For a copy, see <https://opensource.org/licenses/MIT>.

from bark.core.world.opendrive import *
from bark.core.world import *
from bark.core.geometry import *
from bark.core.runtime import PyRuntime


class Runtime(PyRuntime):
  def __init__(self,
               step_time,
               viewer,
               scenario_generator=None,
               render=False):
    self._step_time = step_time
    self._viewer = viewer
    self._scenario_generator = scenario_generator
    self._scenario_idx = None
    self._scenario = None
    self._render = render
    self._reset_has_been_called = False

  def reset(self, scenario=None):
    if scenario:
      self._scenario = scenario
    else:
      self._scenario, self._scenario_idx = \
        self._scenario_generator.get_next_scenario()
    self._world = self._scenario.GetWorldState()
    self._reset_has_been_called = True
    self._viewer.Reset()

  def step(self):
    assert(self._reset_has_been_called==True)
    self._world.Step(self._step_time)
    if self._render:
      self.render()

  def render(self):
    # self._viewer.clear()
    self._viewer.drawWorld(
      self._world,
      self._scenario._eval_agent_ids,
      scenario_idx=self._scenario_idx)
    self._viewer.clear()

  def run(self, steps):
    for step_count in range(steps):
      self.Step()